@@ -13,7 +13,7 @@ typedef struct bl_conf_write {
 
 } bl_conf_write_t;
 
-int bl_set_sys_conf_dir(const char *dir);
+void bl_set_sys_conf_dir(const char *dir);
 
 char *bl_get_sys_rc_path(const char *rcfile);
 
@@ -23,7 +23,7 @@ bl_conf_write_t *bl_conf_write_open(char *name);
 
 int bl_conf_io_write(bl_conf_write_t *conf, const char *key, const char *val);
 
-int bl_conf_write_close(bl_conf_write_t *conf);
+void bl_conf_write_close(bl_conf_write_t *conf);
 
 int bl_conf_io_read(bl_file_t *from, char **key, char **val);
 
