@@ -46,101 +46,40 @@ static char *sys_lang_country = NULL;
 
 #ifndef USE_WIN32API
 static lang_codeset_table_t lang_codeset_table[] = {
-    {
-     "en", "ISO8859-1",
-    },
-    {
-     "da", "ISO8859-1",
-    },
-    {
-     "de", "ISO8859-1",
-    },
-    {
-     "fi", "ISO8859-1",
-    },
-    {
-     "fr", "ISO8859-1",
-    },
-    {
-     "is", "ISO8859-1",
-    },
-    {
-     "it", "ISO8859-1",
-    },
-    {
-     "nl", "ISO8859-1",
-    },
-    {
-     "no", "ISO8859-1",
-    },
-    {
-     "pt", "ISO8859-1",
-    },
-    {
-     "sv", "ISO8859-1",
-    },
-    {
-     "cs", "ISO8859-2",
-    },
-    {
-     "hr", "ISO8859-2",
-    },
-    {
-     "hu", "ISO8859-2",
-    },
-    {
-     "la", "ISO8859-2",
-    },
-    {
-     "lt", "ISO8859-2",
-    },
-    {
-     "pl", "ISO8859-2",
-    },
-    {
-     "sl", "ISO8859-2",
-    },
-    {
-     "el", "ISO8859-7",
-    },
-    {
-     "ru", "KOI8-R",
-    },
-    {
-     "uk", "KOI8-U",
-    },
-    {
-     "vi", "VISCII",
-    },
-    {
-     "th", "TIS-620",
-    },
-    {
-     "ja", "eucJP",
-    },
-    {
-     "ko", "eucKR",
-    },
-    {
-     "zh_CN", "eucCN",
-    },
-    {
-     "zh_TW", "Big5",
-    },
-    {
-     "zh_HK", "Big5HKSCS",
-    },
-
+  { "en", "ISO8859-1", },
+  { "da", "ISO8859-1", },
+  { "de", "ISO8859-1", },
+  { "fi", "ISO8859-1", },
+  { "fr", "ISO8859-1", },
+  { "is", "ISO8859-1", },
+  { "it", "ISO8859-1", },
+  { "nl", "ISO8859-1", },
+  { "no", "ISO8859-1", },
+  { "pt", "ISO8859-1", },
+  { "sv", "ISO8859-1", },
+  { "cs", "ISO8859-2", },
+  { "hr", "ISO8859-2", },
+  { "hu", "ISO8859-2", },
+  { "la", "ISO8859-2", },
+  { "lt", "ISO8859-2", },
+  { "pl", "ISO8859-2", },
+  { "sl", "ISO8859-2", },
+  { "el", "ISO8859-7", },
+  { "ru", "KOI8-R", },
+  { "uk", "KOI8-U", },
+  { "vi", "VISCII", },
+  { "th", "TIS-620", },
+  { "ja", "eucJP", },
+  { "ko", "eucKR", },
+  { "zh_CN", "eucCN", },
+  { "zh_TW", "Big5", },
+  { "zh_HK", "Big5HKSCS", },
 };
 #endif
 
 static alias_codeset_table_t alias_codeset_table[] = {
-    {
-     "EUC", "ja_JP.EUC", "eucJP",
-    },
-    {
-     "EUC", "ko_KR.EUC", "eucKR",
-    },
+  { "EUC", "ja_JP.EUC", "eucJP", },
+  { "EUC", "ko_KR.EUC", "eucKR", },
 };
 
 /* --- global functions --- */
@@ -149,18 +88,8 @@ int bl_locale_init(const char *locale) {
   char *locale_p;
   int result;
 
-  if (sys_locale) {
-    if (locale && strcmp(locale, sys_locale) == 0) {
-      return 1;
-    } else {
-      free(sys_locale);
-      sys_locale = NULL;
-    }
-  }
-
-  if (sys_lang_country) {
-    free(sys_lang_country);
-    sys_lang_country = NULL;
+  if (sys_locale && locale && strcmp(locale, sys_locale) == 0) {
+    return 1;
   }
 
   if ((locale = setlocale(LC_CTYPE, locale)) == NULL) {
@@ -168,33 +97,38 @@ int bl_locale_init(const char *locale) {
     bl_warn_printf(BL_DEBUG_TAG " setlocale() failed.\n");
 #endif
 
-    result = 0;
-
     if (sys_locale) {
       /* restoring locale info. nothing is changed. */
-
       setlocale(LC_CTYPE, sys_locale);
 
       return 0;
-    } else {
-      /* sys_locale is NULL */
+    } else if ((locale = getenv("LC_ALL")) == NULL && (locale = getenv("LC_CTYPE")) == NULL &&
+               (locale = getenv("LANG")) == NULL) {
+      /* nothing is changed */
 
-      if ((locale = getenv("LC_ALL")) == NULL && (locale = getenv("LC_CTYPE")) == NULL &&
-          (locale = getenv("LANG")) == NULL) {
-        /* nothing is changed */
-
-        return 0;
-      }
+      return 0;
     }
-  } else {
-    /*
-     * If external library calls setlocale(), this 'locale' variable
-     * can be free'ed, so strdup() shoule be called.
-     */
-    sys_locale = strdup(locale);
-    result = 1;
+
+    result = 0;
   }
 
+  if (sys_locale) {
+    free(sys_locale);
+    sys_locale = NULL;
+  }
+
+  if (sys_lang_country) {
+    free(sys_lang_country);
+    sys_lang_country = NULL;
+  }
+
+  /*
+   * If external library calls setlocale(), this 'locale' variable
+   * can be free'ed, so strdup() shoule be called.
+   */
+  sys_locale = strdup(locale);
+  result = 1;
+
   if ((locale_p = sys_lang_country = strdup(locale)) == NULL) {
     sys_locale = NULL;
 
@@ -244,7 +178,7 @@ int bl_locale_init(const char *locale) {
   return result;
 }
 
-int bl_locale_final(void) {
+void bl_locale_final(void) {
   if (sys_locale) {
     free(sys_locale);
     sys_locale = NULL;
@@ -254,8 +188,6 @@ int bl_locale_final(void) {
     free(sys_lang_country);
     sys_lang_country = NULL;
   }
-
-  return 1;
 }
 
 char *bl_get_locale(void) {
