@@ -113,7 +113,7 @@ bl_utmp_t bl_utmp_new(const char *tty, const char *host, int pty_fd) {
 
 #ifdef USE_UTMPX
   memcpy(ut.ut_host, host, BL_MIN(sizeof(ut.ut_host), strlen(host)));
-#if !defined(__FreeBSD__) && !defined(__APPLE__)
+#if !defined(__FreeBSD__) && !defined(__APPLE__) && !defined(__CYGWIN__) && !defined(__MSYS__)
   ut.ut_session = getsid(0);
 #endif
 #endif
@@ -168,7 +168,7 @@ bl_utmp_t bl_utmp_new(const char *tty, const char *host, int pty_fd) {
   return utmp;
 }
 
-int bl_utmp_delete(bl_utmp_t utmp) {
+void bl_utmp_destroy(bl_utmp_t utmp) {
 #ifdef USE_UTMPX
   struct utmpx ut;
 #else
@@ -211,6 +211,4 @@ int bl_utmp_delete(bl_utmp_t utmp) {
   bl_priv_change_egid(getgid());
 
   free(utmp);
-
-  return 1;
 }
