@@ -59,7 +59,7 @@
 #include <fcntl.h>
 #include <pobl/bl_unistd.h>
 #include <pobl/bl_mem.h> /* malloc/alloca/free */
-#include <pobl/bl_str.h> /* bl_snprintf/bl_str_alloca_dup/bl_str_sep*/
+#include <pobl/bl_str.h> /* bl_snprintf/strdup/bl_str_sep */
 #include <pobl/bl_dlfcn.h> /* bl_dl_open() */
 #include <pobl/bl_file.h>
 #include <pobl/bl_slist.h>
@@ -238,7 +238,7 @@ static int is_conf_file(char *file_name) {
  * module related functions
  */
 
-static aux_module_t *load_module(char *file_name) {
+static aux_module_t *load_module(char *filename) {
   aux_module_t *module = NULL;
   aux_info_t *aux_info;
   aux_dir_t *aux_dir;
@@ -254,19 +254,20 @@ static aux_module_t *load_module(char *file_name) {
   char *dirpath;
   size_t len;
 
-  if (!is_valid_path(file_name)) {
+  if (!is_valid_path(filename)) {
     return NULL;
   }
 
   /* eliminate leading "./" */
-  if (strncmp(file_name, "./", 2) == 0) {
-    file_name += 2;
+  if (strncmp(filename, "./", 2) == 0) {
+    filename += 2;
   }
 
   /*
    * TODO: bl_dirname()
    */
-  if (!(dirname = bl_str_alloca_dup(file_name)) || !(basename = strrchr(dirname, '/'))) {
+  if (!(dirname = alloca(strlen(filename) + 1)) ||
+      !(basename = strrchr(strcpy(dirname, filename), '/'))) {
     return NULL;
   }
 
@@ -323,7 +324,7 @@ static aux_module_t *load_module(char *file_name) {
     goto error;
   }
 
-  module->file_name = strdup(file_name);
+  module->file_name = strdup(filename);
   module->handle = dl_handle;
   module->entries = NULL;
   module->num_entries = 0;
@@ -1381,17 +1382,17 @@ void aux_init(IIIMCF_handle iiimcf_handle, ui_im_export_syms_t *export_syms,
           char *aux_name;
           char *mod_name;
 
-          if (*line == '#' || *line == '\n') {
+          if (len == 0 || *line == '#') {
+            /* empty line or comment out */
             continue;
           }
 
-          line[ len - 1] = '\0';
-
           while (*line == ' ' || *line == '\t') {
             line ++;
           }
+
           aux_name = bl_str_sep(&line, " ");
-          if (!aux_name || !line) {
+          if (!line) {
             continue;
           }
 
@@ -1490,7 +1491,7 @@ void aux_quit(void) {
   filter_info_list = NULL;
 
   if (conv_iso88591) {
-    (*conv_iso88591->delete)(conv_iso88591);
+    (*conv_iso88591->destroy)(conv_iso88591);
     conv_iso88591 = NULL;
   }
 
@@ -1668,7 +1669,7 @@ void aux_unset_focus(aux_t *aux) {
   }
 }
 
-void aux_delete(aux_t *aux) {
+void aux_destroy(aux_t *aux) {
   aux_id_info_t *info = id_info_list;
   aux_im_data_t *im_data;
 
