@@ -3,7 +3,7 @@
 #include <stdio.h>
 
 #include <pobl/bl_mem.h>  /* malloc/alloca/free */
-#include <pobl/bl_str.h>  /* bl_str_alloca_dup bl_str_sep bl_snprintf*/
+#include <pobl/bl_str.h>  /* strdup/bl_str_sep/bl_snprintf */
 #include <pobl/bl_util.h> /* DIGIT_STR_LEN */
 #include <pobl/bl_conf_io.h>
 
@@ -114,7 +114,7 @@ static void preedit_add(im_skk_t *skk, wchar wch) {
   skk->preedit[skk->preedit_len++] = ch;
 }
 
-static void preedit_delete(im_skk_t *skk) { --skk->preedit_len; }
+static void preedit_destroy(im_skk_t *skk) { --skk->preedit_len; }
 
 static void candidate_clear(im_skk_t *skk);
 
@@ -270,7 +270,7 @@ candidate:
     return;
   } else if (candidateword_len == 0 && (candidateword_len = strlen(candidateword)) == 0) {
     if (skk->im.stat_screen) {
-      (*skk->im.stat_screen->delete)(skk->im.stat_screen);
+      (*skk->im.stat_screen->destroy)(skk->im.stat_screen);
       skk->im.stat_screen = NULL;
     }
   } else {
@@ -337,35 +337,23 @@ static int insert_char(im_skk_t *skk, u_char key_char) {
       {0xa4ab, 0xa4ad, 0xa4af, 0xa4b1, 0xa4b3}, /* c */
       {0xa4c0, 0xa4c2, 0xa4c5, 0xa4c7, 0xa4c9}, /* d */
       {0xa4e3, 0xa4a3, 0xa4e5, 0xa4a7, 0xa4e7}, /* xy/dh */
-      {
-       0, 0, 0xa4d5, 0, 0,
-      },                                        /* f */
+      {0, 0, 0xa4d5, 0, 0,},                    /* f */
       {0xa4ac, 0xa4ae, 0xa4b0, 0xa4b2, 0xa4b4}, /* g */
       {0xa4cf, 0xa4d2, 0xa4d5, 0xa4d8, 0xa4db}, /* h */
       {0xa4e3, 0, 0xa4e5, 0xa4a7, 0xa4e7},      /* ch/sh */
-      {
-       0, 0xa4b8, 0, 0, 0,
-      },                                        /* j */
+      {0, 0xa4b8, 0, 0, 0,},                    /* j */
       {0xa4ab, 0xa4ad, 0xa4af, 0xa4b1, 0xa4b3}, /* k */
       {0xa4a1, 0xa4a3, 0xa4a5, 0xa4a7, 0xa4a9}, /* l */
       {0xa4de, 0xa4df, 0xa4e0, 0xa4e1, 0xa4e2}, /* m */
       {0xa4ca, 0xa4cb, 0xa4cc, 0xa4cd, 0xa4ce}, /* n */
-      {
-       0, 0, 0, 0, 0,
-      },
+      {0, 0, 0, 0, 0,},
       {0xa4d1, 0xa4d4, 0xa4d7, 0xa4da, 0xa4dd}, /* p */
-      {
-       0, 0, 0, 0, 0,
-      },
+      {0, 0, 0, 0, 0,},
       {0xa4e9, 0xa4ea, 0xa4eb, 0xa4ec, 0xa4ed}, /* r */
       {0xa4b5, 0xa4b7, 0xa4b9, 0xa4bb, 0xa4bd}, /* s */
       {0xa4bf, 0xa4c1, 0xa4c4, 0xa4c6, 0xa4c8}, /* t */
-      {
-       0, 0, 0, 0, 0,
-      },
-      {
-       0, 0, 0, 0, 0,
-      },
+      {0, 0, 0, 0, 0,},
+      {0, 0, 0, 0, 0,},
       {0xa4ef, 0xa4f0, 0, 0xa4f1, 0xa4f2},      /* w */
       {0xa4a1, 0xa4a3, 0xa4a5, 0xa4a7, 0xa4a9}, /* x */
       {0xa4e4, 0, 0xa4e6, 0, 0xa4e8},           /* y */
@@ -385,7 +373,7 @@ static int insert_char(im_skk_t *skk, u_char key_char) {
   wchar wch;
 
   if (skk->dan) {
-    preedit_delete(skk);
+    preedit_destroy(skk);
   }
 
   if (key_char == 'a' || key_char == 'i' || key_char == 'u' || key_char == 'e' || key_char == 'o') {
@@ -578,15 +566,15 @@ static void insert_alphabet_full(im_skk_t *skk, u_char key_char) {
  * methods of ui_im_t
  */
 
-static void delete(ui_im_t *im) {
+static void destroy(ui_im_t *im) {
   im_skk_t *skk;
 
   skk = (im_skk_t*)im;
 
-  (*skk->parser_term->delete)(skk->parser_term);
+  (*skk->parser_term->destroy)(skk->parser_term);
 
   if (skk->conv) {
-    (*skk->conv->delete)(skk->conv);
+    (*skk->conv->destroy)(skk->conv);
   }
 
   free(skk->status[HIRAGANA]);
@@ -610,7 +598,7 @@ static void delete(ui_im_t *im) {
   }
 
 #ifdef IM_SKK_DEBUG
-  bl_debug_printf(BL_DEBUG_TAG " An object was deleted. ref_count: %d\n", ref_count);
+  bl_debug_printf(BL_DEBUG_TAG " An object was destroyed. ref_count: %d\n", ref_count);
 #endif
 }
 
@@ -884,7 +872,7 @@ static int key_event(ui_im_t *im, u_char key_char, KeySym ksym, XKeyEvent *event
         if (skk->preedit_len == 1) {
           preedit_clear(skk);
         } else {
-          preedit_delete(skk);
+          preedit_destroy(skk);
         }
       }
     } else if (skk->is_editing_new_word) {
@@ -1205,13 +1193,13 @@ ui_im_t *im_skk_new(u_int64_t magic, vt_char_encoding_t term_encoding,
       skk->status[mode] = strdup(buf);
     }
 
-    (*parser->delete)(parser);
+    (*parser->destroy)(parser);
   }
 
   /*
    * set methods of ui_im_t
    */
-  skk->im.delete = delete;
+  skk->im.destroy = destroy;
   skk->im.key_event = key_event;
   skk->im.switch_mode = switch_mode;
   skk->im.is_active = is_active;
@@ -1229,11 +1217,11 @@ ui_im_t *im_skk_new(u_int64_t magic, vt_char_encoding_t term_encoding,
 error:
   if (skk) {
     if (skk->parser_term) {
-      (*skk->parser_term->delete)(skk->parser_term);
+      (*skk->parser_term->destroy)(skk->parser_term);
     }
 
     if (skk->conv) {
-      (*skk->conv->delete)(skk->conv);
+      (*skk->conv->destroy)(skk->conv);
     }
 
     free(skk);
