@@ -38,8 +38,14 @@
 #include <uim-helper.h>
 #include <uim-im-switcher.h>
 
+#if UIM_VERSION_REQUIRE(1, 5, 0)
+#include <uim-scm.h>
+#else
+#define uim_scm_is_initialized() (initialized)
+#endif
+
 #include <pobl/bl_mem.h>    /* malloc/alloca/free */
-#include <pobl/bl_str.h>    /* bl_str_alloca_dup bl_str_sep bl_snprintf*/
+#include <pobl/bl_str.h>    /* strdup/bl_str_sep/bl_snprintf */
 #include <pobl/bl_locale.h> /* bl_get_locale */
 #include <pobl/bl_slist.h>
 
@@ -111,7 +117,7 @@ static void update_stat_screen(im_uim_t *uim, int mode_changed) {
   if (!uim->im.cand_screen) {
     if (uim->mode == 0 /* direct input */ /* || uim->im.preedit.filled_len == 0 */) {
       if (uim->im.stat_screen) {
-        (*uim->im.stat_screen->delete)(uim->im.stat_screen);
+        (*uim->im.stat_screen->destroy)(uim->im.stat_screen);
         uim->im.stat_screen = NULL;
       }
 
@@ -511,7 +517,7 @@ static void preedit_clear(void *ptr) {
   uim = (im_uim_t *)ptr;
 
   if (uim->im.preedit.chars) {
-    (*syms->vt_str_delete)(uim->im.preedit.chars, uim->im.preedit.num_chars);
+    (*syms->vt_str_destroy)(uim->im.preedit.chars, uim->im.preedit.num_chars);
     uim->im.preedit.chars = NULL;
   }
 
@@ -592,7 +598,7 @@ static void preedit_pushback(void *ptr, int attr, const char *_str) {
       bl_warn_printf(BL_DEBUG_TAG " realloc failed.\n");
 #endif
 
-      (*syms->vt_str_delete)(uim->im.preedit.chars, uim->im.preedit.num_chars);
+      (*syms->vt_str_destroy)(uim->im.preedit.chars, uim->im.preedit.num_chars);
       uim->im.preedit.chars = NULL;
       uim->im.preedit.num_chars = 0;
       uim->im.preedit.filled_len = 0;
@@ -701,7 +707,7 @@ static void candidate_activate(void *p, int num, int limit) {
                                 uim->im.preedit.segment_offset, &x, &y);
 
   if (uim->im.stat_screen) {
-    (*uim->im.stat_screen->delete)(uim->im.stat_screen);
+    (*uim->im.stat_screen->destroy)(uim->im.stat_screen);
     uim->im.stat_screen = NULL;
   }
 
@@ -722,7 +728,7 @@ static void candidate_activate(void *p, int num, int limit) {
   }
 
   if (!(*uim->im.cand_screen->init)(uim->im.cand_screen, num, limit)) {
-    (*uim->im.cand_screen->delete)(uim->im.cand_screen);
+    (*uim->im.cand_screen->destroy)(uim->im.cand_screen);
     uim->im.cand_screen = NULL;
 
     return;
@@ -844,7 +850,7 @@ static void candidate_deactivate(void *p) {
   uim = (im_uim_t *)p;
 
   if (uim->im.cand_screen) {
-    (*uim->im.cand_screen->delete)(uim->im.cand_screen);
+    (*uim->im.cand_screen->destroy)(uim->im.cand_screen);
     uim->im.cand_screen = NULL;
   }
 }
@@ -853,7 +859,7 @@ static void candidate_deactivate(void *p) {
  * methods of ui_im_t
  */
 
-static void delete(ui_im_t *im) {
+static void destroy(ui_im_t *im) {
   im_uim_t *uim;
 
   uim = (im_uim_t *)im;
@@ -863,13 +869,13 @@ static void delete(ui_im_t *im) {
   }
 
   if (uim->parser_uim) {
-    (*uim->parser_uim->delete)(uim->parser_uim);
+    (*uim->parser_uim->destroy)(uim->parser_uim);
   }
 
-  (*uim->parser_term->delete)(uim->parser_term);
+  (*uim->parser_term->destroy)(uim->parser_term);
 
   if (uim->conv) {
-    (*uim->conv->delete)(uim->conv);
+    (*uim->conv->destroy)(uim->conv);
   }
 
   uim_release_context(uim->context);
@@ -877,7 +883,7 @@ static void delete(ui_im_t *im) {
   ref_count--;
 
 #ifdef IM_UIM_DEBUG
-  bl_debug_printf(BL_DEBUG_TAG " An object was deleted. ref_count: %d\n", ref_count);
+  bl_debug_printf(BL_DEBUG_TAG " An object was destroyed. ref_count: %d\n", ref_count);
 #endif
 
   bl_slist_remove(uim_list, uim);
@@ -890,9 +896,10 @@ static void delete(ui_im_t *im) {
     uim_helper_close_client_fd(helper_fd);
     helper_fd = -1;
 
-    uim_quit();
-
-    initialized = 0;
+    if (initialized > 0) {
+      uim_quit();
+      initialized = 0;
+    }
   }
 }
 
@@ -1181,7 +1188,7 @@ static void helper_commit_string(u_char *str /* UTF-8? */
   }
 
   if (!(parser_utf8 = (*syms->vt_char_encoding_parser_new)(VT_UTF8))) {
-    (*conv->delete)(conv);
+    (*conv->destroy)(conv);
     return;
   }
 
@@ -1202,8 +1209,8 @@ static void helper_commit_string(u_char *str /* UTF-8? */
                                                filled_len);
   }
 
-  (*parser_utf8->delete)(parser_utf8);
-  (*conv->delete)(conv);
+  (*parser_utf8->destroy)(parser_utf8);
+  (*conv->destroy)(conv);
 }
 
 static void helper_read_handler(void) {
@@ -1275,13 +1282,16 @@ ui_im_t *im_uim_new(u_int64_t magic, vt_char_encoding_t term_encoding,
 #define RESTORE_LOCALE
 #endif
 
-  if (!initialized) {
+  if (!uim_scm_is_initialized()) {
 #ifdef RESTORE_LOCALE
     /*
      * Workaround against make_locale() of m17nlib.
      */
     char *cur_locale;
-    cur_locale = bl_str_alloca_dup(bl_get_locale());
+
+    if ((cur_locale = alloca(strlen(bl_get_locale()) + 1))) {
+      strcpy(cur_locale, bl_get_locale());
+    }
 #endif
 
     if (uim_init() == -1) {
@@ -1305,6 +1315,8 @@ ui_im_t *im_uim_new(u_int64_t magic, vt_char_encoding_t term_encoding,
     syms = export_syms;
 
     initialized = 1;
+  } else if (!initialized) {
+    initialized = -1;
   }
 
   /*
@@ -1317,10 +1329,15 @@ ui_im_t *im_uim_new(u_int64_t magic, vt_char_encoding_t term_encoding,
   }
 
   if ((engine == NULL) || (strlen(engine) == 0)) {
-    engine = (char *)uim_get_default_im_name(bl_get_locale());
-    /* The returned string's storage is invalidated when we
-     * call uim next, so we need to make a copy.  */
-    engine = bl_str_alloca_dup(engine);
+    /*
+     * The returned string's storage is invalidated when we
+     * call uim next, so we need to make a copy.
+     */
+    char *p = (char *)uim_get_default_im_name(bl_get_locale());
+
+    if ((engine = alloca(strlen(p) + 1))) {
+      strcpy(engine, p);
+    }
   }
 
   if (!find_engine(engine, &encoding_name)) {
@@ -1389,7 +1406,7 @@ ui_im_t *im_uim_new(u_int64_t magic, vt_char_encoding_t term_encoding,
   /*
    * set methods of ui_im_t
    */
-  uim->im.delete = delete;
+  uim->im.destroy = destroy;
   uim->im.key_event = key_event;
   uim->im.switch_mode = switch_mode;
   uim->im.is_active = is_active;
@@ -1415,9 +1432,8 @@ error:
     helper_fd = -1;
   }
 
-  if (initialized && ref_count == 0) {
+  if (initialized > 0 && ref_count == 0) {
     uim_quit();
-
     initialized = 0;
   }
 
@@ -1425,15 +1441,15 @@ error:
 
   if (uim) {
     if (uim->parser_uim) {
-      (*uim->parser_uim->delete)(uim->parser_uim);
+      (*uim->parser_uim->destroy)(uim->parser_uim);
     }
 
     if (uim->parser_term) {
-      (*uim->parser_term->delete)(uim->parser_term);
+      (*uim->parser_term->destroy)(uim->parser_term);
     }
 
     if (uim->conv) {
-      (*uim->conv->delete)(uim->conv);
+      (*uim->conv->destroy)(uim->conv);
     }
 
     free(uim);
@@ -1448,13 +1464,18 @@ im_info_t *im_uim_get_info(char *locale, char *encoding) {
   im_info_t *result = NULL;
   uim_context u;
   int i;
+  int uim_init_done;
 
-  if (uim_init() == -1) {
+  if (uim_scm_is_initialized()) {
+    uim_init_done = 0;
+  } else if (uim_init() == -1) {
 #ifdef DEBUG
     bl_warn_printf(BL_DEBUG_TAG " failed to initialize uim.");
 #endif
 
     return NULL;
+  } else {
+    uim_init_done = 1;
   }
 
   if (!(u = uim_create_context(NULL, "UTF-8", NULL, NULL, NULL, NULL))) {
@@ -1499,7 +1520,9 @@ im_info_t *im_uim_get_info(char *locale, char *encoding) {
 
   uim_release_context(u);
 
-  uim_quit();
+  if (uim_init_done) {
+    uim_quit();
+  }
 
   result->id = strdup("uim");
   result->name = strdup("uim");
@@ -1512,7 +1535,9 @@ error:
     uim_release_context(u);
   }
 
-  uim_quit();
+  if (uim_init_done) {
+    uim_quit();
+  }
 
   if (result) {
     if (result->args) {
