@@ -3,7 +3,7 @@
 #include <stdio.h>
 
 #include <pobl/bl_mem.h>  /* malloc/alloca/free */
-#include <pobl/bl_str.h>  /* bl_str_alloca_dup bl_str_sep bl_snprintf*/
+#include <pobl/bl_str.h>  /* strdup/bl_str_sep/bl_snprintf */
 #include <pobl/bl_util.h> /* DIGIT_STR_LEN */
 
 #include <ui_im.h>
@@ -85,7 +85,7 @@ static void wchar_parser_set_str(ef_parser_t *parser, u_char *str, size_t size)
   parser->is_eos = 0;
 }
 
-static void wchar_parser_delete(ef_parser_t *parser) { free(parser); }
+static void wchar_parser_destroy(ef_parser_t *parser) { free(parser); }
 
 static int wchar_parser_next_char(ef_parser_t *parser, ef_char_t *ch) {
   wchar wch;
@@ -135,7 +135,7 @@ static ef_parser_t *wchar_parser_new(void) {
 
   parser->init = ef_parser_init;
   parser->set_str = wchar_parser_set_str;
-  parser->delete = wchar_parser_delete;
+  parser->destroy = wchar_parser_destroy;
   parser->next_char = wchar_parser_next_char;
 
   return parser;
@@ -254,7 +254,7 @@ candidate:
     return;
   } else if (candidateword_len == 0) {
     if (wnn->im.stat_screen) {
-      (*wnn->im.stat_screen->delete)(wnn->im.stat_screen);
+      (*wnn->im.stat_screen->destroy)(wnn->im.stat_screen);
       wnn->im.stat_screen = NULL;
     }
   } else {
@@ -328,35 +328,23 @@ static int insert_char(im_wnn_t *wnn, u_char key_char) {
       {0xa4ab, 0xa4ad, 0xa4af, 0xa4b1, 0xa4b3}, /* c */
       {0xa4c0, 0xa4c2, 0xa4c5, 0xa4c7, 0xa4c9}, /* d */
       {0xa4e3, 0xa4a3, 0xa4e5, 0xa4a7, 0xa4e7}, /* xy */
-      {
-       0, 0, 0xa4d5, 0, 0,
-      },                                        /* f */
+      {0, 0, 0xa4d5, 0, 0,},                    /* f */
       {0xa4ac, 0xa4ae, 0xa4b0, 0xa4b2, 0xa4b4}, /* g */
       {0xa4cf, 0xa4d2, 0xa4d5, 0xa4d8, 0xa4db}, /* h */
       {0xa4e3, 0, 0xa4e5, 0xa4a7, 0xa4e7},      /* ch/sh */
-      {
-       0, 0xa4b8, 0, 0, 0,
-      },                                        /* j */
+      {0, 0xa4b8, 0, 0, 0,},                    /* j */
       {0xa4ab, 0xa4ad, 0xa4af, 0xa4b1, 0xa4b3}, /* k */
       {0xa4a1, 0xa4a3, 0xa4a5, 0xa4a7, 0xa4a9}, /* l */
       {0xa4de, 0xa4df, 0xa4e0, 0xa4e1, 0xa4e2}, /* m */
       {0xa4ca, 0xa4cb, 0xa4cc, 0xa4cd, 0xa4ce}, /* n */
-      {
-       0, 0, 0, 0, 0,
-      },
+      {0, 0, 0, 0, 0,},
       {0xa4d1, 0xa4d4, 0xa4d7, 0xa4da, 0xa4dd}, /* p */
-      {
-       0, 0, 0, 0, 0,
-      },
+      {0, 0, 0, 0, 0,},
       {0xa4e9, 0xa4ea, 0xa4eb, 0xa4ec, 0xa4ed}, /* r */
       {0xa4b5, 0xa4b7, 0xa4b9, 0xa4bb, 0xa4bd}, /* s */
       {0xa4bf, 0xa4c1, 0xa4c4, 0xa4c6, 0xa4c8}, /* t */
-      {
-       0, 0, 0, 0, 0,
-      },
-      {
-       0, 0, 0, 0, 0,
-      },
+      {0, 0, 0, 0, 0,},
+      {0, 0, 0, 0, 0,},
       {0xa4ef, 0xa4f0, 0, 0xa4f1, 0xa4f2},      /* w */
       {0xa4a1, 0xa4a3, 0xa4a5, 0xa4a7, 0xa4a9}, /* x */
       {0xa4e4, 0, 0xa4e6, 0, 0xa4e8},           /* y */
@@ -580,16 +568,16 @@ static int fix(im_wnn_t *wnn) {
  * methods of ui_im_t
  */
 
-static void delete(ui_im_t *im) {
+static void destroy(ui_im_t *im) {
   im_wnn_t *wnn;
   struct wnn_buf *buf;
 
   wnn = (im_wnn_t*)im;
 
-  (*wnn->parser_term->delete)(wnn->parser_term);
+  (*wnn->parser_term->destroy)(wnn->parser_term);
 
   if (wnn->conv) {
-    (*wnn->conv->delete)(wnn->conv);
+    (*wnn->conv->destroy)(wnn->conv);
   }
 
   buf = wnn->convbuf->wnn;
@@ -601,11 +589,11 @@ static void delete(ui_im_t *im) {
   ref_count--;
 
 #ifdef IM_WNN_DEBUG
-  bl_debug_printf(BL_DEBUG_TAG " An object was deleted. ref_count: %d\n", ref_count);
+  bl_debug_printf(BL_DEBUG_TAG " An object was destroyed. ref_count: %d\n", ref_count);
 #endif
 
   if (ref_count == 0) {
-    (*parser_wchar->delete)(parser_wchar);
+    (*parser_wchar->destroy)(parser_wchar);
     parser_wchar = NULL;
   }
 }
@@ -878,7 +866,7 @@ ui_im_t *im_wnn_new(u_int64_t magic, vt_char_encoding_t term_encoding,
   /*
    * set methods of ui_im_t
    */
-  wnn->im.delete = delete;
+  wnn->im.destroy = destroy;
   wnn->im.key_event = key_event;
   wnn->im.switch_mode = switch_mode;
   wnn->im.is_active = is_active;
@@ -896,18 +884,18 @@ ui_im_t *im_wnn_new(u_int64_t magic, vt_char_encoding_t term_encoding,
 error:
   if (ref_count == 0) {
     if (parser_wchar) {
-      (*parser_wchar->delete)(parser_wchar);
+      (*parser_wchar->destroy)(parser_wchar);
       parser_wchar = NULL;
     }
   }
 
   if (wnn) {
     if (wnn->parser_term) {
-      (*wnn->parser_term->delete)(wnn->parser_term);
+      (*wnn->parser_term->destroy)(wnn->parser_term);
     }
 
     if (wnn->conv) {
-      (*wnn->conv->delete)(wnn->conv);
+      (*wnn->conv->destroy)(wnn->conv);
     }
 
     buf = wnn->convbuf->wnn;
