@@ -1,4 +1,6 @@
-/* indian.h - This is the header file used for incorporating Indian Script 
+/* -*- c-basic-offset:2; tab-width:2; indent-tabs-mode:nil -*- */
+
+/* indian.h - This is the header file used for incorporating Indian Script
  * support in X Windows Applications.
  * Copyright (C) 1999, Naoshad A. Mehta, Rudrava Roy
  */
@@ -10,15 +12,14 @@
 
 /* Structure to hold key maps */
 struct a2i_tabl {
-	char *ascii;
-	char *iscii;
+  char *ascii;
+  char *iscii;
 };
 
 /* Structure to hold font map */
-struct tabl
-{
-	char *iscii;
-	char *font;
+struct tabl {
+  char *iscii;
+  char *font;
 };
 
 /* Private function prototypes */
@@ -26,7 +27,6 @@ struct tabl
 char *binsearch(struct tabl *, int, char *);
 char *split(struct tabl *, char *, int);
 
-
 /* Public function prototypes */
 
 int iscii2font(struct tabl *, char *, char *, int);
