@@ -55,6 +55,8 @@ typedef im_info_t *(*im_get_info_func_t)(char *, char *);
 
 /* --- static variables --- */
 
+static char *cur_locale;
+
 static im_type_t im_type;
 static im_type_t cur_im_type;
 
@@ -187,7 +189,13 @@ static int get_im_info(char *locale, char *encoding) {
     if (!(p = strstr(d->d_name, "im-"))) continue;
 
     end = p + strlen(p);
-    if (strcmp(gui, "fb") == 0 || strcmp(gui, "console") == 0 || strcmp(gui, "wayland") == 0) {
+    if (strcmp(gui, "sdl2") == 0) {
+      end -= 5;
+      if (strcmp(end, "-sdl2") != 0) {
+        continue;
+      }
+    } else if (strcmp(gui, "fb") == 0 || strcmp(gui, "console") == 0 ||
+               strcmp(gui, "wayland") == 0) {
       end -= 3;
       if (strcmp(end, *gui == 'w' ? "-wl" : "-fb") != 0) {
         continue;
@@ -339,7 +347,7 @@ static GtkWidget *xim_widget_new(const char *xim_name, const char *xim_locale,
 
   num_xims = size + 1;
 
-  bl_map_delete(xim_locale_table);
+  bl_map_destroy(xim_locale_table);
 
   /*
    * create widgets
@@ -643,9 +651,17 @@ static gint button_im_checked(GtkWidget *widget, gpointer data) {
 
 /* --- global functions --- */
 
+void mc_im_init(void) {
+  char *encoding = mc_get_str_value("encoding");
+
+  cur_locale = mc_get_str_value("locale"); /* XXX leaked */
+
+  get_im_info(cur_locale, encoding);
+
+  free(encoding);
+}
+
 GtkWidget *mc_im_config_widget_new(void) {
-  char *cur_locale = NULL;
-  char *encoding = NULL;
   char *xim_name = NULL;
   char *xim_locale = NULL;
   char *value;
@@ -659,11 +675,6 @@ GtkWidget *mc_im_config_widget_new(void) {
   GtkWidget *radio;
   GSList *group;
 
-  cur_locale = mc_get_str_value("locale");
-  encoding = mc_get_str_value("encoding");
-
-  get_im_info(cur_locale, encoding);
-
   value = mc_get_str_value("input_method");
 
   im_name = bl_str_sep(&value, ":");
@@ -703,8 +714,6 @@ GtkWidget *mc_im_config_widget_new(void) {
       im_opt_widget[i] = im_widget_new(i, index == i ? value : NULL, cur_locale);
   }
 
-  free(cur_locale);
-  free(encoding);
   free(im_name);
 
   frame = gtk_frame_new(_("Input Method"));
