@@ -544,5 +544,7 @@ KeySym XStringToKeysym(char *str);
 #undef COMPOSE_DECSP_FONT
 #undef USE_REAL_VERTICAL_FONT
 #undef NO_DISPLAY_FD
+#undef FLICK_SCROLL
+#undef UIWINDOW_SUPPORTS_PREEDITING
 
 #endif
