@@ -13,7 +13,7 @@
 #endif
 #include <pobl/bl_debug.h>
 #include <pobl/bl_mem.h> /* realloc */
-#include <pobl/bl_str.h> /* bl_str_sep/bl_str_to_int/memset/strncasecmp */
+#include <pobl/bl_str.h> /* bl_str_to_int/memset/strncasecmp */
 #include <vt_char.h>     /* UTF_MAX_SIZE */
 
 #ifdef USE_OT_LAYOUT
@@ -76,6 +76,18 @@ static int parse_fc_font_name(
 
   *font_family = font_name;
 
+  p = font_name + strlen(font_name) - 1;
+  if ('0' <= *p && *p <= '9') {
+    do {
+      p--;
+    } while ('0' <= *p && *p <= '9');
+
+    if (*p == ':' && bl_str_to_uint(percent, p + 1)) {
+      /* Parsing ":[Percentage]" */
+      *p = '\0';
+    }
+  }
+
   p = font_name;
   while (1) {
     if (*p == '\\' && *(p + 1)) {
@@ -88,18 +100,6 @@ static int parse_fc_font_name(
 
       *font_name = '\0';
 
-      break;
-    } else if (*p == ':') {
-      /* Parsing ":[Percentage]" */
-
-      *font_name = '\0';
-
-      if (!bl_str_to_uint(percent, p + 1)) {
-#ifdef DEBUG
-        bl_warn_printf(BL_DEBUG_TAG " Percentage(%s) is illegal.\n", p + 1);
-#endif
-      }
-
       break;
     }
 
@@ -142,43 +142,26 @@ static int parse_fc_font_name(
           int slant;
 
         } styles[] = {
-            /*
-             * Portable styles.
-             */
-            /* slant */
-            {
-             "italic", 0, FC_SLANT_ITALIC,
-            },
-            /* weight */
-            {
-             "bold", FC_WEIGHT_BOLD, 0,
-            },
+          /*
+           * Portable styles.
+           */
+          /* slant */
+          { "italic", 0, FC_SLANT_ITALIC, },
+          /* weight */
+          { "bold", FC_WEIGHT_BOLD, 0, },
 
-            /*
-             * Hack for styles which can be returned by
-             * gtk_font_selection_dialog_get_font_name().
-             */
-            /* slant */
-            {
-             "oblique", 0, FC_SLANT_OBLIQUE,
-            },
-            /* weight */
-            {
-             "light", /* e.g. "Bookman Old Style Light" */
-             FC_WEIGHT_LIGHT, 0,
-            },
-            {
-             "semi-bold", FC_WEIGHT_DEMIBOLD, 0,
-            },
-            {
-             "heavy", /* e.g. "Arial Black Heavy" */
-             FC_WEIGHT_BLACK, 0,
-            },
-            /* other */
-            {
-             "semi-condensed", /* XXX This style is ignored. */
-             0, 0,
-            },
+          /*
+           * Hack for styles which can be returned by
+           * gtk_font_selection_dialog_get_font_name().
+           */
+          /* slant */
+          { "oblique", 0, FC_SLANT_OBLIQUE, },
+          /* weight */
+          { "light", /* e.g. "Bookman Old Style Light" */ FC_WEIGHT_LIGHT, 0, },
+          { "semi-bold", FC_WEIGHT_DEMIBOLD, 0, },
+          { "heavy", /* e.g. "Arial Black Heavy" */ FC_WEIGHT_BLACK, 0, },
+          /* other */
+          { "semi-condensed", /* XXX This style is ignored. */ 0, 0, },
         };
 
         for (count = 0; count < sizeof(styles) / sizeof(styles[0]); count++) {
@@ -273,13 +256,30 @@ static FcPattern *fc_pattern_create(char *family,                        /* can
                                     int aa_opt) {
   FcPattern *pattern;
 
-  if (!(pattern = FcPatternCreate())) {
+  if (family) {
+    /*
+     * XXX
+     * FcNameParse() unexpectedly separates "BN-TTDurga" to "BN" and "TTDurga".
+     * '-' must be escaped by '\' before calling FcNameParse().
+     */
+    if (strchr(family, '-')) {
+      if (strchr(family, ':')) { /* BN-TTDurga:style=... */
+        bl_msg_printf("Failed to parse %s.\n", family);
+      }
+    } else if ((pattern = FcNameParse(family))) {
+      goto created;
+    }
+  }
+
+  if ((pattern = FcPatternCreate())) {
+    if (family) {
+      FcPatternAddString(pattern, FC_FAMILY, family);
+    }
+  } else {
     return NULL;
   }
 
-  if (family) {
-    FcPatternAddString(pattern, FC_FAMILY, family);
-  }
+created:
   FcPatternAddDouble(pattern, fc_size_type, size);
   if (weight >= 0) {
     FcPatternAddInteger(pattern, FC_WEIGHT, weight);
@@ -370,7 +370,8 @@ static XftFont *xft_font_open(ui_font_t *font, char *family, /* can be NULL */
     return NULL;
   }
 
-#if 1
+  /* ui_window_xft_draw_string8() calls FT_Select_Charmap(FT_ENCODING_APPLE_ROMAN) */
+#if 0
   if (is_iscii) {
     FT_Face face;
     int count;
@@ -525,7 +526,8 @@ static cairo_scaled_font_t *cairo_font_open(ui_font_t *font, char *family, /* ca
     goto error2;
   }
 
-#if 1
+  /* ui_window_cairo_draw_string8() calls FT_Select_Charmap(FT_ENCODING_APPLE_ROMAN) */
+#if 0
   if (IS_ISCII(cs)) {
     FT_Face face;
     int count;
@@ -685,7 +687,7 @@ static FcCharSet *get_cached_charset(const char *family) {
 }
 
 #if 0
-static void delete_charset_chache(void) {
+static void destroy_charset_chache(void) {
   u_int count;
   for (count = 0; count < charset_cache_size; count++) {
     free(charset_cache[count].family);
@@ -910,13 +912,10 @@ static int fc_set_font(ui_font_t *font, const char *fontname, u_int fontsize,
     double fontsize_d;
     u_int percent;
 
-    if ((p = bl_str_alloca_dup(fontname)) == NULL) {
-#ifdef DEBUG
-      bl_warn_printf(BL_DEBUG_TAG " alloca() failed.\n");
-#endif
-
+    if ((p = alloca(strlen(fontname) + 1)) == NULL) {
       return 0;
     }
+    strcpy(p, fontname);
 
     fontsize_d = (double)fontsize;
     percent = 0;
@@ -1356,7 +1355,7 @@ void cairo_unset_font(ui_font_t *font) {
   }
 
 #if 0
-  delete_charset_cache();
+  destroy_charset_cache();
 #endif
 }
 
