@@ -211,7 +211,7 @@ static u_int total_min_height(ui_window_t *win) {
   return min_height;
 }
 
-static u_int total_width_inc(ui_window_t *win) {
+static u_int max_width_inc(ui_window_t *win) {
   u_int count;
   u_int width_inc;
 
@@ -225,7 +225,7 @@ static u_int total_width_inc(ui_window_t *win) {
        * XXX
        * we should calculate least common multiple of width_inc and sub_inc.
        */
-      if ((sub_inc = total_width_inc(win->children[count])) > width_inc) {
+      if ((sub_inc = max_width_inc(win->children[count])) > width_inc) {
         width_inc = sub_inc;
       }
     }
@@ -234,7 +234,7 @@ static u_int total_width_inc(ui_window_t *win) {
   return width_inc;
 }
 
-static u_int total_height_inc(ui_window_t *win) {
+static u_int max_height_inc(ui_window_t *win) {
   u_int count;
   u_int height_inc;
 
@@ -248,7 +248,7 @@ static u_int total_height_inc(ui_window_t *win) {
        * XXX
        * we should calculate least common multiple of width_inc and sub_inc.
        */
-      if ((sub_inc = total_height_inc(win->children[count])) > height_inc) {
+      if ((sub_inc = max_height_inc(win->children[count])) > height_inc) {
         height_inc = sub_inc;
       }
     }
@@ -803,7 +803,7 @@ static void poll_event(void) {
 #if 0
       if (modify_resize(disp->display->width, disp->display->height, &width, &height,
                         total_min_width(disp->roots[0]), total_min_height(disp->roots[0]),
-                        total_width_inc(disp->roots[0]), total_height_inc(disp->roots[0]), 1)) {
+                        max_width_inc(disp->roots[0]), max_height_inc(disp->roots[0]), 1)) {
         SDL_SetWindowSize(disp->display->window, width, height);
       } else
 #endif
@@ -1032,7 +1032,10 @@ int ui_display_show_root(ui_display_t *disp, ui_window_t *root, int x, int y, in
     disp->display->width = disp->width;
     disp->display->height = disp->height;
   }
-  init_display(disp->display, root->app_name, x, y, hint);
+
+  if (!init_display(disp->display, root->app_name, x, y, hint)) {
+    return 0;
+  }
 
   ui_window_show(root, hint);
 
@@ -1364,7 +1367,6 @@ void ui_display_set_title(ui_display_t *disp, const u_char *name) {
   SDL_SetWindowTitle(disp->display->window, name);
 }
 
-#ifdef USE_WIN32API
 void ui_display_trigger_pty_read(void) {
   SDL_Event ev;
 
@@ -1372,7 +1374,6 @@ void ui_display_trigger_pty_read(void) {
   ev.type = pty_event_type;
   SDL_PushEvent(&ev);
 }
-#endif
 
 #ifdef USE_BG_TEXTURE
 void ui_display_set_bg_color(ui_display_t *disp, u_long bg) {
