@@ -8,6 +8,7 @@
 #include <pobl/bl_locale.h> /* bl_get_lang */
 #include <pobl/bl_args.h>
 #include <pobl/bl_util.h>   /* BL_INT_TO_STR */
+#include <pobl/bl_unistd.h> /* bl_setenv */
 
 #include <vt_term_manager.h>
 #include "ui_selection_encoding.h"
@@ -105,7 +106,7 @@ void ui_prepare_for_main_config(bl_conf_t *conf) {
   bl_conf_add_opt(conf, '9', "crfg", 0, "cursor_fg_color", "cursor foreground color");
   bl_conf_add_opt(conf, '0', "crbg", 0, "cursor_bg_color", "cursor background color");
 #if !defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_XFT) || defined(USE_TYPE_CAIRO) || \
-  defined(USE_FREETYPE)
+  defined(USE_FREETYPE) || defined(USE_WIN32GUI)
   bl_conf_add_opt(conf, 'A', "aa", 1, "use_anti_alias",
                   "forcibly use anti alias font by using Xft or cairo");
 #endif
@@ -328,6 +329,10 @@ void ui_prepare_for_main_config(bl_conf_t *conf) {
                   "emoji directory or file path [~/.mlterm/emoji]");
   bl_conf_add_opt(conf, '\0', "lew", 0, "local_echo_wait",
                   "time (msec) to keep local echo mode [250]");
+  bl_conf_add_opt(conf, '\0', "sr", 1, "scroll_on_resizing",
+                  "scroll screen on resizing [false]");
+  bl_conf_add_opt(conf, '\0', "recvdir", 0, "receive_directory",
+                  "directory to save received files [~/.mlterm/recv]");
 #ifdef USE_IM_CURSOR_COLOR
   bl_conf_add_opt(conf, '\0', "imcolor", 0, "im_cursor_color",
                   "cursor color when input method is activated. [false]");
@@ -337,7 +342,7 @@ void ui_prepare_for_main_config(bl_conf_t *conf) {
 
 void ui_main_config_init(ui_main_config_t *main_config, bl_conf_t *conf, int argc, char **argv) {
   char *value;
-  char *invalid_msg = "%s %s is not valid.\n";
+  char *invalid_msg = "%s=%s is invalid.\n";
 
   memset(main_config, 0, sizeof(ui_main_config_t));
 
@@ -356,12 +361,18 @@ void ui_main_config_init(ui_main_config_t *main_config, bl_conf_t *conf, int arg
 #endif
   {
     main_config->disp_name = default_display;
+#ifdef USE_XLIB
+    /* In case of starting mlterm with "-j genuine" option without X server. */
+    if (getenv("DISPLAY") == NULL) {
+      bl_setenv("DISPLAY", ":0", 0);
+    }
+#endif
   }
 
   if ((value = bl_conf_get_value(conf, "fontsize")) == NULL) {
     main_config->font_size = 16;
   } else if (!bl_str_to_uint(&main_config->font_size, value)) {
-    bl_msg_printf(invalid_msg, "font size", value);
+    bl_msg_printf(invalid_msg, "fontsize", value);
 
     /* default value is used. */
     main_config->font_size = 16;
@@ -449,7 +460,7 @@ void ui_main_config_init(ui_main_config_t *main_config, bl_conf_t *conf, int arg
     if (bl_str_to_uint(&size, value)) {
       main_config->step_in_changing_font_size = size;
     } else {
-      bl_msg_printf(invalid_msg, "step in changing font size", value);
+      bl_msg_printf(invalid_msg, "step_in_changing_font_size", value);
     }
   }
 
@@ -470,7 +481,7 @@ void ui_main_config_init(ui_main_config_t *main_config, bl_conf_t *conf, int arg
 #endif
 
 #if !defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_XFT) || defined(USE_TYPE_CAIRO) || \
-  defined(USE_FREETYPE)
+  defined(USE_FREETYPE) || defined(USE_WIN32GUI)
   if ((value = bl_conf_get_value(conf, "use_anti_alias"))) {
     if (strcmp(value, "true") == 0) {
       main_config->font_present |= FONT_AA;
@@ -628,7 +639,7 @@ void ui_main_config_init(ui_main_config_t *main_config, bl_conf_t *conf, int arg
     if (bl_str_to_int(&size, value)) {
       main_config->line_space = size;
     } else {
-      bl_msg_printf(invalid_msg, "line space", value);
+      bl_msg_printf(invalid_msg, "line_space", value);
     }
   }
 
@@ -638,7 +649,7 @@ void ui_main_config_init(ui_main_config_t *main_config, bl_conf_t *conf, int arg
     if (bl_str_to_uint(&size, value)) {
       main_config->letter_space = size;
     } else {
-      bl_msg_printf(invalid_msg, "letter space", value);
+      bl_msg_printf(invalid_msg, "letter_space", value);
     }
   }
 
@@ -653,7 +664,7 @@ void ui_main_config_init(ui_main_config_t *main_config, bl_conf_t *conf, int arg
     } else if (bl_str_to_uint(&size, value)) {
       main_config->num_log_lines = size;
     } else {
-      bl_msg_printf(invalid_msg, "log size", value);
+      bl_msg_printf(invalid_msg, "logsize", value);
     }
   }
 
@@ -665,7 +676,7 @@ void ui_main_config_init(ui_main_config_t *main_config, bl_conf_t *conf, int arg
     if (bl_str_to_uint(&size, value)) {
       main_config->tab_size = size;
     } else {
-      bl_msg_printf(invalid_msg, "tab size", value);
+      bl_msg_printf(invalid_msg, "tabsize", value);
     }
   }
 
@@ -682,12 +693,16 @@ void ui_main_config_init(ui_main_config_t *main_config, bl_conf_t *conf, int arg
   }
 
   if ((value = bl_conf_get_value(conf, "big5_buggy"))) {
-    ui_set_big5_selection_buggy(strcmp(value, "true") == 0);
+    int flag = true_or_false(value);
+
+    if (flag != -1) {
+      ui_set_big5_selection_buggy(flag);
+    }
   }
 
   main_config->use_mdi = 1;
 
-#if defined(USE_QUARTZ) || defined(USE_BEOS)
+#if !defined(USE_QUARTZ) && !defined(USE_BEOS)
   if ((value = bl_conf_get_value(conf, "use_mdi"))) {
     if (strcmp(value, "false") == 0) {
       main_config->use_mdi = 0;
@@ -793,7 +808,7 @@ void ui_main_config_init(ui_main_config_t *main_config, bl_conf_t *conf, int arg
     if (bl_str_to_uint(&col_size_of_width_a, value)) {
       main_config->col_size_of_width_a = col_size_of_width_a;
     } else {
-      bl_msg_printf(invalid_msg, "col size of width a", value);
+      bl_msg_printf(invalid_msg, "col_size_of_width_a", value);
     }
   }
 
@@ -826,7 +841,7 @@ void ui_main_config_init(ui_main_config_t *main_config, bl_conf_t *conf, int arg
     if (bl_str_to_uint(&brightness, value)) {
       main_config->brightness = brightness;
     } else {
-      bl_msg_printf(invalid_msg, "shade ratio", value);
+      bl_msg_printf(invalid_msg, "brightness", value);
     }
   }
 
@@ -838,7 +853,7 @@ void ui_main_config_init(ui_main_config_t *main_config, bl_conf_t *conf, int arg
     if (bl_str_to_uint(&contrast, value)) {
       main_config->contrast = contrast;
     } else {
-      bl_msg_printf(invalid_msg, "contrast ratio", value);
+      bl_msg_printf(invalid_msg, "contrast", value);
     }
   }
 
@@ -850,7 +865,7 @@ void ui_main_config_init(ui_main_config_t *main_config, bl_conf_t *conf, int arg
     if (bl_str_to_uint(&gamma, value)) {
       main_config->gamma = gamma;
     } else {
-      bl_msg_printf(invalid_msg, "gamma ratio", value);
+      bl_msg_printf(invalid_msg, "gamma", value);
     }
   }
 
@@ -874,7 +889,7 @@ void ui_main_config_init(ui_main_config_t *main_config, bl_conf_t *conf, int arg
     if (bl_str_to_uint(&fade_ratio, value) && fade_ratio <= 100) {
       main_config->fade_ratio = fade_ratio;
     } else {
-      bl_msg_printf(invalid_msg, "fade ratio", value);
+      bl_msg_printf(invalid_msg, "fade_ratio", value);
     }
   }
 
@@ -957,7 +972,11 @@ void ui_main_config_init(ui_main_config_t *main_config, bl_conf_t *conf, int arg
   }
 
   if ((value = bl_conf_get_value(conf, "use_urgent_bell"))) {
-    ui_set_use_urgent_bell(strcmp(value, "true") == 0);
+    int flag = true_or_false(value);
+
+    if (flag != -1) {
+      ui_set_use_urgent_bell(flag);
+    }
   }
 
   main_config->use_vertical_cursor = 1;
@@ -990,11 +1009,19 @@ void ui_main_config_init(ui_main_config_t *main_config, bl_conf_t *conf, int arg
   }
 
   if ((value = bl_conf_get_value(conf, "exit_backscroll_by_pty"))) {
-    ui_exit_backscroll_by_pty(strcmp(value, "true") == 0);
+    int flag = true_or_false(value);
+
+    if (flag != -1) {
+      ui_exit_backscroll_by_pty(flag);
+    }
   }
 
   if ((value = bl_conf_get_value(conf, "allow_change_shortcut"))) {
-    ui_allow_change_shortcut(strcmp(value, "true") == 0);
+    int flag = true_or_false(value);
+
+    if (flag != -1) {
+      ui_allow_change_shortcut(flag);
+    }
   }
 
   if ((value = bl_conf_get_value(conf, "icon_path"))) {
@@ -1053,7 +1080,7 @@ void ui_main_config_init(ui_main_config_t *main_config, bl_conf_t *conf, int arg
     if (bl_str_to_uint(&parent_window, value)) {
       main_config->parent_window = parent_window;
     } else {
-      bl_msg_printf(invalid_msg, "parent window", value);
+      bl_msg_printf(invalid_msg, "parent_window", value);
     }
   }
 
@@ -1083,15 +1110,25 @@ void ui_main_config_init(ui_main_config_t *main_config, bl_conf_t *conf, int arg
   }
 
   if ((value = bl_conf_get_value(conf, "ssh_x11_forwarding"))) {
-    main_config->use_x11_forwarding = (strcmp(value, "true") == 0);
+    if (strcmp(value, "true") == 0) {
+      main_config->use_x11_forwarding = 1;
+    }
   }
 
   if ((value = bl_conf_get_value(conf, "allow_scp"))) {
-    vt_set_use_scp_full(strcmp(value, "true") == 0);
+    int flag = true_or_false(value);
+
+    if (flag != -1) {
+      vt_set_use_scp_full(flag);
+    }
   }
 
   if ((value = bl_conf_get_value(conf, "ssh_auto_reconnect"))) {
-    vt_pty_ssh_set_use_auto_reconnect(strcmp(value, "true") == 0);
+    int flag = true_or_false(value);
+
+    if (flag != -1) {
+      vt_pty_ssh_set_use_auto_reconnect(flag);
+    }
   }
 #endif
 
@@ -1169,7 +1206,7 @@ void ui_main_config_init(ui_main_config_t *main_config, bl_conf_t *conf, int arg
     if (bl_str_to_int(&size, value)) {
       main_config->underline_offset = size;
     } else {
-      bl_msg_printf(invalid_msg, "underline offset", value);
+      bl_msg_printf(invalid_msg, "underline_offset", value);
     }
   }
 
@@ -1179,7 +1216,7 @@ void ui_main_config_init(ui_main_config_t *main_config, bl_conf_t *conf, int arg
     if (bl_str_to_int(&size, value)) {
       main_config->baseline_offset = size;
     } else {
-      bl_msg_printf(invalid_msg, "baseline offset", value);
+      bl_msg_printf(invalid_msg, "baseline_offset", value);
     }
   }
 
@@ -1188,11 +1225,19 @@ void ui_main_config_init(ui_main_config_t *main_config, bl_conf_t *conf, int arg
   }
 
   if ((value = bl_conf_get_value(conf, "regard_uri_as_word"))) {
-    vt_set_regard_uri_as_word(strcmp(value, "true") == 0);
+    int flag = true_or_false(value);
+
+    if (flag != -1) {
+      vt_set_regard_uri_as_word(flag);
+    }
   }
 
   if ((value = bl_conf_get_value(conf, "use_clipboard"))) {
-    ui_set_use_clipboard_selection(strcmp(value, "true") == 0);
+    int flag = true_or_false(value);
+
+    if (flag != -1) {
+      ui_set_use_clipboard_selection(flag);
+    }
   }
 
 #ifndef __ANDROID__
@@ -1225,7 +1270,11 @@ void ui_main_config_init(ui_main_config_t *main_config, bl_conf_t *conf, int arg
   }
 
   if ((value = bl_conf_get_value(conf, "use_alt_buffer"))) {
-    vt_set_use_alt_buffer(strcmp(value, "true") == 0);
+    int flag = true_or_false(value);
+
+    if (flag != -1) {
+      vt_set_use_alt_buffer(flag);
+    }
   }
 
   main_config->use_ansi_colors = 1;
@@ -1249,7 +1298,11 @@ void ui_main_config_init(ui_main_config_t *main_config, bl_conf_t *conf, int arg
   }
 
   if ((value = bl_conf_get_value(conf, "leftward_double_drawing"))) {
-    ui_set_use_leftward_double_drawing(strcmp(value, "true") == 0);
+    int flag = true_or_false(value);
+
+    if (flag != -1) {
+      ui_set_use_leftward_double_drawing(flag);
+    }
   }
 
   if ((value = bl_conf_get_value(conf, "working_directory"))) {
@@ -1297,8 +1350,11 @@ void ui_main_config_init(ui_main_config_t *main_config, bl_conf_t *conf, int arg
 #ifdef USE_GRF
   if ((value = bl_conf_get_value(conf, "separate_wall_picture"))) {
     extern int separate_wall_picture;
+    int flag = true_or_false(value);
 
-    separate_wall_picture = (strcmp(value, "true") == 0 ? 1 : 0);
+    if (flag != -1) {
+      separate_wall_picture = flag;
+    }
   }
 #endif
 
@@ -1310,19 +1366,27 @@ void ui_main_config_init(ui_main_config_t *main_config, bl_conf_t *conf, int arg
 
 #if defined(__ANDROID__) && defined(USE_LIBSSH2)
   if ((value = bl_conf_get_value(conf, "start_with_local_pty"))) {
-    start_with_local_pty = (strcmp(value, "true") == 0);
+    int flag = true_or_false(value);
+
+    if (flag != -1) {
+      start_with_local_pty = flag;
+    }
   }
 #endif
 
   if ((value = bl_conf_get_value(conf, "trim_trailing_newline_in_pasting"))) {
-    if (strcmp(value, "true") == 0) {
-      ui_set_trim_trailing_newline_in_pasting(1);
+    int flag = true_or_false(value);
+
+    if (flag != -1) {
+      ui_set_trim_trailing_newline_in_pasting(flag);
     }
   }
 
   if ((value = bl_conf_get_value(conf, "broadcast"))) {
-    if (strcmp(value, "true") == 0) {
-      vt_set_broadcasting(1);
+    int flag = true_or_false(value);
+
+    if (flag != -1) {
+      vt_set_broadcasting(flag);
     }
   }
 
@@ -1336,6 +1400,18 @@ void ui_main_config_init(ui_main_config_t *main_config, bl_conf_t *conf, int arg
     ui_emoji_set_path(value);
   }
 
+  if ((value = bl_conf_get_value(conf, "scroll_on_resizing"))) {
+    int flag = true_or_false(value);
+
+    if (flag != -1) {
+      vt_set_scroll_on_resizing(flag);
+    }
+  }
+
+  if ((value = bl_conf_get_value(conf, "receive_directory"))) {
+    vt_set_recv_dir(value);
+  }
+
 #ifdef USE_IM_CURSOR_COLOR
   if ((value = bl_conf_get_value(conf, "im_cursor_color"))) {
     if (*value) {
