@@ -234,10 +234,25 @@ static int open_pty_intern(vt_term_t *term, char *cmd_path, char **cmd_argv,
 #endif
     if (!ui_connect_dialog(&uri, &pass, &exec_cmd, &privkey, &x11_fwd, display, window,
                            main_config.default_server)) {
-      bl_msg_printf("Connect dialog is canceled.\n");
+#ifdef DEBUG
+      bl_debug_printf(BL_DEBUG_TAG " Connect dialog is canceled.\n");
+#endif
+
+      /*
+       * open_screen_intern() calls close_screen_intern() if this fuction
+       * returns 0, then this thread will be terminated on Haiku.
+       * (If mlclient --serv ... is executed and the connect dialog is cancelled,
+       *  the screen where mlclient is executed also exits.)
+       * So don't return 0 even if the connect dialog is cancelled.
+       *
+       * (vt_term_open_pty() below never returns 0 because OPEN_PTY_ASYNC
+       *  is defined on Haiku.)
+       */
+#ifndef USE_BEOS
       if (vt_get_all_terms(NULL) > 1) {
         return 0;
       }
+#endif
     } else if ((uri_dup = alloca(strlen(uri) + 1))) {
       bl_parse_uri(NULL, &user, &host, &port, NULL, &encoding, strcpy(uri_dup, uri));
 
@@ -1071,9 +1086,9 @@ static int prev_screen(void *self, ui_screen_t *screen) {
   }
 }
 
-static int resize_screen(void *self, ui_screen_t *screen, int horizontal, int step) {
+static int resize_screen(void *self, ui_screen_t *screen, int horizontal, const char *size) {
   if (UI_SCREEN_TO_LAYOUT(screen)) {
-    return ui_layout_resize(UI_SCREEN_TO_LAYOUT(screen), screen, horizontal, step);
+    return ui_layout_resize(UI_SCREEN_TO_LAYOUT(screen), screen, horizontal, size);
   } else {
     return 0;
   }
