@@ -3,11 +3,19 @@
 #ifndef ___UI_H__
 #define ___UI_H__
 
+#ifndef COMPAT_LIBVTE
+#define XDG_SHELL_V6
+#endif
+
 #include <wayland-client.h>
 #include <wayland-client-protocol.h>
 #include <wayland-cursor.h>
 #include <wayland-egl.h>
 #include <xkbcommon/xkbcommon.h>
+#include "gtk-primary-selection.h"
+#ifdef XDG_SHELL_V6
+#include "xdg-shell-unstable-v6-client-protocol.h"
+#endif
 
 #ifdef USE_FREETYPE
 #include <pobl/bl_types.h> /* u_int32_t etc */
@@ -74,9 +82,6 @@ typedef struct {
   struct wl_keyboard *keyboard;
   struct wl_pointer *pointer;
 
-  struct wl_data_device_manager *data_device_manager;
-  struct wl_data_device *data_device;
-
   struct ui_xkb {
     struct xkb_context *ctx;
     struct xkb_keymap *keymap;
@@ -101,20 +106,39 @@ typedef struct {
   XKeyEvent prev_kev;
   u_int kbd_repeat_count;
 
+  struct wl_data_device_manager *data_device_manager;
+  struct wl_data_device *data_device;
   struct wl_data_offer *dnd_offer;
   struct wl_surface *data_surface;
   uint32_t dnd_action;
   struct wl_data_offer *sel_offer;
+  char *sel_offer_mime;
   struct wl_data_source *sel_source;
+
+  struct gtk_primary_selection_device_manager *xsel_device_manager;
+  struct gtk_primary_selection_device *xsel_device;
+  struct gtk_primary_selection_offer *xsel_offer;
+  char *xsel_offer_mime;
+  struct gtk_primary_selection_source *xsel_source;
+
   int32_t sel_fd;
   uint32_t serial;
 
   int ref_count;
 
+  /*
+   * These members should be placed at the end of this structure because
+   * ui_*.c except ui_display.c refers this structure correctly
+   * if COMPAT_LIBVTE is defined or not.
+   */
 #ifdef COMPAT_LIBVTE
   struct wl_subcompositor *subcompositor;
+  int is_xdg_shell; /* for sway 1.0 */
 #else
   struct wl_shell *shell;
+#ifdef XDG_SHELL_V6
+  struct zxdg_shell_v6 *xdg_shell;
+#endif
 #endif
 
 } ui_wlserv_t;
@@ -152,6 +176,11 @@ typedef struct {
 
   struct ui_display *parent;
 
+  /*
+   * These members should be placed at the end of this structure because
+   * ui_*.c except ui_display.c refers this structure correctly
+   * if COMPAT_LIBVTE is defined or not.
+   */
 #ifdef COMPAT_LIBVTE
   struct wl_surface *parent_surface;
   struct wl_subsurface *subsurface;
@@ -159,6 +188,11 @@ typedef struct {
   int y;
 #else
   struct wl_shell_surface *shell_surface;
+#ifdef XDG_SHELL_V6
+  struct zxdg_surface_v6 *xdg_surface;
+  struct zxdg_toplevel_v6 *xdg_toplevel;
+  int xdg_surface_configured;
+#endif
 #endif
 
 } Display;
@@ -212,6 +246,7 @@ typedef struct _XFontStruct {
   int32_t glyph_width_bytes;
 
   unsigned char width;
+  /* Width of full width characters or max width of half width characters. */
   unsigned char width_full;
   unsigned char height;
   unsigned char ascent;
