@@ -269,7 +269,7 @@ static void seat_capabilities(void *data, struct wl_seat *seat, enum wl_seat_cap
 
 static void seat_name(void *data, struct wl_seat *seat, const char *name) {
 #ifdef __DEBUG
-  bl_debug_printf("SEAT name %s\n", name);
+  bl_debug_printf("seat_name %s\n", name);
 #endif
 }
 
@@ -278,6 +278,16 @@ static const struct wl_seat_listener seat_listener = {
   seat_name,
 };
 
+#ifdef XDG_SHELL_V6
+static void xdg_shell_ping(void *data, struct zxdg_shell_v6 *xdg_shell, uint32_t serial) {
+  zxdg_shell_v6_pong(xdg_shell, serial);
+}
+
+static const struct zxdg_shell_v6_listener xdg_shell_listener = {
+  xdg_shell_ping
+};
+#endif
+
 static void registry_global(void *data, struct wl_registry *registry, uint32_t name,
                             const char *interface, uint32_t version) {
   ui_wlserv_t *wlserv = data;
@@ -289,23 +299,48 @@ static void registry_global(void *data, struct wl_registry *registry, uint32_t n
   else if (strcmp(interface, "wl_subcompositor") == 0) {
     wlserv->subcompositor = wl_registry_bind(registry, name, &wl_subcompositor_interface, 1);
   }
+  /* For sway 1.0 which doesn't support wl_surface. (supports only xdg-shell) */
+  else if (strcmp(interface, "wl_shell") == 0) {
+    wlserv->is_xdg_shell = -1; /* not xdg_shell */
+  } else if (strcmp(interface, "zxdg_shell_v6") == 0) {
+    if (wlserv->is_xdg_shell == 0) {
+      wlserv->is_xdg_shell = 1;
+    }
+  }
 #else
-  else if(strcmp(interface, "wl_shell") == 0) {
-    wlserv->shell = wl_registry_bind(registry, name,
-                                     &wl_shell_interface, 1);
+  else if (strcmp(interface, "wl_shell") == 0) {
+    wlserv->shell = wl_registry_bind(registry, name, &wl_shell_interface, 1);
+
+#ifdef XDG_SHELL_V6
+    if (wlserv->xdg_shell) {
+      zxdg_shell_v6_destroy(wlserv->xdg_shell);
+      wlserv->xdg_shell = NULL;
+    }
+#endif
+  }
+#ifdef XDG_SHELL_V6
+  else if (strcmp(interface, "zxdg_shell_v6") == 0) {
+    if (wlserv->shell == NULL) {
+      wlserv->xdg_shell = wl_registry_bind(registry, name, &zxdg_shell_v6_interface, 1);
+      zxdg_shell_v6_add_listener(wlserv->xdg_shell, &xdg_shell_listener, NULL);
+    }
   }
 #endif
-  else if(strcmp(interface, "wl_shm") == 0) {
+#endif
+  else if (strcmp(interface, "wl_shm") == 0) {
     wlserv->shm = wl_registry_bind(registry, name, &wl_shm_interface, 1);
-  } else if(strcmp(interface, "wl_seat") == 0) {
+  } else if (strcmp(interface, "wl_seat") == 0) {
     wlserv->seat = wl_registry_bind(registry, name,
                                     &wl_seat_interface, 4); /* 4 is for keyboard_repeat_info */
     wl_seat_add_listener(wlserv->seat, &seat_listener, wlserv);
-  } else if(strcmp(interface, "wl_output") == 0) {
+  } else if (strcmp(interface, "wl_output") == 0) {
     wlserv->output = wl_registry_bind(registry, name, &wl_output_interface, 1);
   } else if (strcmp(interface, "wl_data_device_manager") == 0) {
     wlserv->data_device_manager = wl_registry_bind(registry, name,
                                                    &wl_data_device_manager_interface, 3);
+  } else if (strcmp(interface, "gtk_primary_selection_device_manager") == 0) {
+    wlserv->xsel_device_manager =
+      wl_registry_bind(registry, name, &gtk_primary_selection_device_manager_interface, 1);
   }
 #ifdef __DEBUG
   else {
@@ -371,7 +406,9 @@ static void auto_repeat(void) {
   u_int count;
 
   for (count = 0; count < num_wlservs; count++) {
-    if (wlservs[count]->prev_kev.type && --wlservs[count]->kbd_repeat_wait == 0) {
+    /* If prev_kev.type > 0, kbd_repeat_wait is always greater than 0. */
+    if (wlservs[count]->prev_kev.type && /* wlservs[count]->kbd_repeat_wait > 0 && */
+        --wlservs[count]->kbd_repeat_wait == 0) {
       if (wlservs[count]->kbd_repeat_count++ == 2) {
         /*
          * Selecting next candidate of ibus lookup table by repeating space key
@@ -383,7 +420,11 @@ static void auto_repeat(void) {
 
       wlservs[count]->prev_kev.time += kbd_repeat_N;
       receive_key_event(wlservs[count], &wlservs[count]->prev_kev);
-      wlservs[count]->kbd_repeat_wait = (kbd_repeat_N + KEY_REPEAT_UNIT / 2) / KEY_REPEAT_UNIT;
+      if (kbd_repeat_N < KEY_REPEAT_UNIT / 2) {
+        wlservs[count]->kbd_repeat_wait = 1;
+      } else {
+        wlservs[count]->kbd_repeat_wait = (kbd_repeat_N + KEY_REPEAT_UNIT / 2) / KEY_REPEAT_UNIT;
+      }
     }
   }
 }
@@ -412,7 +453,7 @@ static void keyboard_enter(void *data, struct wl_keyboard *keyboard,
   wlserv->current_kbd_surface = surface;
 
 #ifdef __DEBUG
-  bl_debug_printf("KBD ENTER %p\n", surface);
+  bl_debug_printf("keyboard_enter %p\n", surface);
 #endif
 
   if ((win = search_inputtable_window(NULL, disp->roots[0]))) {
@@ -433,7 +474,7 @@ static void keyboard_leave(void *data, struct wl_keyboard *keyboard,
   ui_window_t *win;
 
 #ifdef __DEBUG
-  bl_debug_printf("KBD LEAVE %p\n", surface);
+  bl_debug_printf("keyboard_leave %p\n", surface);
 #endif
 
 #if 0
@@ -491,11 +532,22 @@ static void keyboard_key(void *data, struct wl_keyboard *keyboard,
     ev.state = wlserv->xkb->mods;
     ev.time = time;
 
-    wlserv->kbd_repeat_wait = (kbd_repeat_1 + KEY_REPEAT_UNIT - 1) / KEY_REPEAT_UNIT;
-    wlserv->prev_kev = ev;
+    if (kbd_repeat_N == 0) {
+      /* disable key repeating */
+      wlserv->kbd_repeat_wait = 0;
+      wlserv->prev_kev.type = 0;
+    } else {
+      if (kbd_repeat_1 == 0) {
+        wlserv->kbd_repeat_wait = 1;
+      } else {
+        wlserv->kbd_repeat_wait = (kbd_repeat_1 + KEY_REPEAT_UNIT - 1) / KEY_REPEAT_UNIT;
+      }
+
+      wlserv->prev_kev = ev;
+    }
 
 #ifdef __DEBUG
-    bl_debug_printf("Receive key %x %x at %d\n", ev.ksym, wlserv->xkb->mods, time);
+    bl_debug_printf("key pressed %x %x at %d\n", ev.ksym, wlserv->xkb->mods, time);
 #endif
   } else if (state_w == WL_KEYBOARD_KEY_STATE_RELEASED) {
 #if 0
@@ -551,11 +603,13 @@ static void keyboard_repeat_info(void *data, struct wl_keyboard *keyboard,
   bl_debug_printf("Repeat info rate %d delay %d.\n", rate, delay);
 #endif
   kbd_repeat_1 = delay;
-#ifdef COMPAT_LIBVTE
-  kbd_repeat_N = (1000 / rate); /* XXX 1000 / rate is too late. */
-#else
-  kbd_repeat_N = (500 / rate); /* XXX 1000 / rate is too late. */
-#endif
+
+  /* The rate of repeating keys in characters per second. 0 disables any repeating. */
+  if (rate == 0) {
+    kbd_repeat_N = 0;
+  } else {
+    kbd_repeat_N = 1000 / rate;
+  }
 }
 
 static const struct wl_keyboard_listener keyboard_listener = {
@@ -588,7 +642,7 @@ static void pointer_enter(void *data, struct wl_pointer *pointer,
   ui_wlserv_t *wlserv = data;
 
 #ifdef __DEBUG
-  bl_debug_printf("POINTER ENTER %p\n", surface);
+  bl_debug_printf("pointer_enter %p\n", surface);
 #endif
 
   wlserv->current_pointer_surface = surface;
@@ -614,7 +668,7 @@ static void pointer_leave(void *data, struct wl_pointer *pointer,
   }
 
 #ifdef __DEBUG
-  bl_debug_printf("POINTER LEAVE %p\n", surface);
+  bl_debug_printf("pointer_leave %p\n", surface);
 #endif
 }
 
@@ -661,7 +715,7 @@ static void pointer_motion(void *data, struct wl_pointer *pointer,
   ui_display_t *disp;
 
 #ifdef ____DEBUG
-  bl_debug_printf("Pointer motion at %d\n", time);
+  bl_debug_printf("pointer_motion (time: %d)\n", time);
 #endif
   disp = surface_to_display(wlserv->current_pointer_surface);
 
@@ -733,7 +787,7 @@ static void pointer_button(void *data, struct wl_pointer *pointer, uint32_t seri
   ui_display_t *disp;
 
 #ifdef __DEBUG
-  bl_debug_printf("Pointer button at %d\n", time);
+  bl_debug_printf("pointer_button (time: %d)\n", time);
 #endif
   disp = surface_to_display(wlserv->current_pointer_surface);
 
@@ -751,10 +805,25 @@ static void pointer_button(void *data, struct wl_pointer *pointer, uint32_t seri
         int state = get_edge_state(disp->display->width, disp->display->height, ev.x, ev.y);
         if (state > 0) {
           if (state == 11) {
-            wl_shell_surface_move(disp->display->shell_surface, wlserv->seat, serial);
+#ifdef XDG_SHELL_V6
+            if (disp->display->xdg_toplevel) {
+              zxdg_toplevel_v6_move(disp->display->xdg_toplevel, wlserv->seat, serial);
+            } else
+#endif
+            {
+              wl_shell_surface_move(disp->display->shell_surface, wlserv->seat, serial);
+            }
           } else {
             disp->display->is_resizing = 1;
-            wl_shell_surface_resize(disp->display->shell_surface, wlserv->seat, serial, state);
+
+#ifdef XDG_SHELL_V6
+            if (disp->display->xdg_toplevel) {
+              zxdg_toplevel_v6_resize(disp->display->xdg_toplevel, wlserv->seat, serial, state);
+            } else
+#endif
+            {
+              wl_shell_surface_resize(disp->display->shell_surface, wlserv->seat, serial, state);
+            }
           }
 
           return;
@@ -816,7 +885,7 @@ static void pointer_axis(void *data, struct wl_pointer *pointer,
   ui_display_t *disp;
 
 #ifdef __DEBUG
-  bl_debug_printf("Pointer axis at %d\n", time);
+  bl_debug_printf("pointer_axis (time: %d)\n", time);
 #endif
   disp = surface_to_display(wlserv->current_pointer_surface);
 
@@ -917,7 +986,7 @@ static void surface_enter(void *data, struct wl_surface *surface, struct wl_outp
   ui_wlserv_t *wlserv = data;
 
 #ifdef __DEBUG
-  bl_debug_printf("SURFACE ENTER %p\n", surface);
+  bl_debug_printf("surface_enter %p\n", surface);
 #endif
 
   wlserv->current_kbd_surface = surface;
@@ -927,7 +996,7 @@ static void surface_leave(void *data, struct wl_surface *surface, struct wl_outp
   ui_wlserv_t *wlserv = data;
 
 #ifdef __DEBUG
-  bl_debug_printf("SURFACE LEAVE %p\n", surface);
+  bl_debug_printf("surface_leave %p\n", surface);
 #endif
 
   if (wlserv->current_kbd_surface == surface) {
@@ -948,7 +1017,7 @@ static int create_shm_buffer(Display *display) {
 
 #ifdef __DEBUG
   bl_debug_printf("Buffer w %d h %d bpp %d\n",
-                  display->width, display->height, display->bytes_per_pixel);
+                  display->width, display->height, display->bytes_per_pixel * 8);
 #endif
 
   display->line_length = display->width * display->bytes_per_pixel;
@@ -968,7 +1037,15 @@ static int create_shm_buffer(Display *display) {
   wl_shm_pool_destroy(pool);
   close(fd);
 
-  wl_surface_attach(display->surface, display->buffer, 0, 0);
+#if defined(COMPAT_LIBVTE)
+  if (display->wlserv->is_xdg_shell != 1)
+#elif defined(XDG_SHELL_V6)
+  /* wl_surface_attach() must be called after xdg_surface_configure() on xdg-shell. */
+  if (display->xdg_surface == NULL)
+#endif
+  {
+    wl_surface_attach(display->surface, display->buffer, 0, 0);
+  }
 
   return 1;
 }
@@ -1015,7 +1092,7 @@ static u_int total_min_height(ui_window_t *win) {
   return min_height;
 }
 
-static u_int total_width_inc(ui_window_t *win) {
+static u_int max_width_inc(ui_window_t *win) {
   u_int count;
   u_int width_inc;
 
@@ -1029,7 +1106,7 @@ static u_int total_width_inc(ui_window_t *win) {
        * XXX
        * we should calculate least common multiple of width_inc and sub_inc.
        */
-      if ((sub_inc = total_width_inc(win->children[count])) > width_inc) {
+      if ((sub_inc = max_width_inc(win->children[count])) > width_inc) {
         width_inc = sub_inc;
       }
     }
@@ -1038,7 +1115,7 @@ static u_int total_width_inc(ui_window_t *win) {
   return width_inc;
 }
 
-static u_int total_height_inc(ui_window_t *win) {
+static u_int max_height_inc(ui_window_t *win) {
   u_int count;
   u_int height_inc;
 
@@ -1052,7 +1129,7 @@ static u_int total_height_inc(ui_window_t *win) {
        * XXX
        * we should calculate least common multiple of width_inc and sub_inc.
        */
-      if ((sub_inc = total_height_inc(win->children[count])) > height_inc) {
+      if ((sub_inc = max_height_inc(win->children[count])) > height_inc) {
         height_inc = sub_inc;
       }
     }
@@ -1174,13 +1251,13 @@ static void shell_surface_configure(void *data, struct wl_shell_surface *shell_s
   ui_display_t *disp = data;
 
 #ifdef __DEBUG
-  bl_debug_printf("Configuring size from w %d h %d to w %d h %d.\n",
+  bl_debug_printf("shell_surface_configure\nConfiguring size from w %d h %d to w %d h %d.\n",
                   disp->display->width, disp->display->height, width, height);
 #endif
 
   if (check_resize(disp->display->width, disp->display->height, &width, &height,
                    total_min_width(disp->roots[0]), total_min_height(disp->roots[0]),
-                   total_width_inc(disp->roots[0]), total_height_inc(disp->roots[0]),
+                   max_width_inc(disp->roots[0]), max_height_inc(disp->roots[0]),
                    disp->display->is_resizing)) {
 #ifdef __DEBUG
     bl_msg_printf("-> modified size w %d h %d\n", width, height);
@@ -1204,18 +1281,124 @@ static const struct wl_shell_surface_listener shell_surface_listener = {
   shell_surface_configure,
   shell_surface_popup_done
 };
+
+#ifdef XDG_SHELL_V6
+static void xdg_surface_configure(void *data, struct zxdg_surface_v6 *xdg_surface,
+                                  uint32_t serial) {
+  ui_display_t *disp = data;
+
+#ifdef __DEBUG
+  bl_debug_printf("xdg_surface_configure\n");
 #endif
 
-static void data_offer_offer(void *data, struct wl_data_offer *offer, const char *type) {
-#ifdef __DEBUG
-  bl_debug_printf("DATA OFFER %s\n", type);
+  zxdg_surface_v6_ack_configure(xdg_surface, serial);
+
+  disp->display->xdg_surface_configured = 1;
+
+  /* See flush_damage() */
+#if 0
+  wl_surface_attach(disp->display->surface, disp->display->buffer, 0, 0);
+  wl_surface_damage(disp->display->surface, 0, 0, disp->display->width, disp->display->height);
+  wl_surface_commit(disp->display->surface);
 #endif
 }
 
+static const struct zxdg_surface_v6_listener xdg_surface_listener = {
+  xdg_surface_configure
+};
+
+static void xdg_toplevel_configure(void *data, struct zxdg_toplevel_v6 *xdg_toplevel,
+                                   int32_t width, int32_t height, struct wl_array *states) {
+  ui_display_t *disp = data;
+#ifdef __DEBUG
+  uint32_t *ps;
+
+  bl_debug_printf("xdg_toplevel_configure: w %d, h %d / states: ", width, height);
+  wl_array_for_each(ps, states) {
+  switch(*ps) {
+    case ZXDG_TOPLEVEL_V6_STATE_MAXIMIZED:
+      bl_msg_printf("MAXIMIZED ");
+      break;
+    case ZXDG_TOPLEVEL_V6_STATE_FULLSCREEN:
+      bl_msg_printf("FULLSCREEN ");
+      break;
+    case ZXDG_TOPLEVEL_V6_STATE_RESIZING:
+      bl_msg_printf("RESIZING ");
+      break;
+    case ZXDG_TOPLEVEL_V6_STATE_ACTIVATED:
+      bl_msg_printf("ACTIVATED ");
+      break;
+    }
+  }
+  bl_msg_printf("\n");
+#endif
+
+#ifdef __DEBUG
+  bl_debug_printf("Configuring size from w %d h %d to w %d h %d.\n",
+                  disp->display->width, disp->display->height, width, height);
+#endif
+
+  if (width == 0) {
+    width = disp->display->width;
+  }
+  if (height == 0) {
+    height = disp->display->height;
+  }
+
+  if (check_resize(disp->display->width, disp->display->height, &width, &height,
+                   total_min_width(disp->roots[0]), total_min_height(disp->roots[0]),
+                   max_width_inc(disp->roots[0]), max_height_inc(disp->roots[0]),
+                   disp->display->is_resizing)) {
+#ifdef __DEBUG
+    bl_msg_printf("-> modified size w %d h %d\n", width, height);
+#endif
+
+    if (rotate_display) {
+      resize_display(disp, height, width, 0);
+    } else {
+      resize_display(disp, width, height, 0);
+    }
+
+    ui_window_resize_with_margin(disp->roots[0], disp->width, disp->height, NOTIFY_TO_MYSELF);
+  }
+}
+
+static void xdg_toplevel_close(void *data, struct zxdg_toplevel_v6 *toplevel) {
+  ui_display_t *disp = data;
+
+  if (disp->roots[0]->window_destroyed) {
+    (*disp->roots[0]->window_destroyed)(disp->roots[0]);
+  }
+}
+
+const struct zxdg_toplevel_v6_listener xdg_toplevel_listener = {
+  xdg_toplevel_configure, xdg_toplevel_close
+};
+#endif
+#endif
+
+static void update_mime(char **mime, char *new_mime) {
+  if (strcmp(new_mime, "text/plain;charset=utf-8") == 0) {
+    *mime = "text/plain;charset=utf-8";
+  } else if (*mime == NULL && strcmp(new_mime, "UTF8_STRING") == 0) {
+    *mime = "UTF8_STRING";
+  }
+}
+
+static void data_offer_offer(void *data, struct wl_data_offer *offer, const char *type) {
+  ui_wlserv_t *wlserv = data;
+
+#ifdef __DEBUG
+  bl_debug_printf("data_offer_offer %s\n", type);
+#endif
+
+  update_mime(&wlserv->sel_offer_mime, type);
+}
+
 static void data_offer_source_action(void *data, struct wl_data_offer *data_offer,
                                      uint32_t source_actions) {
 #ifdef __DEBUG
-  bl_debug_printf("SOURCE ACTION %d\n", source_actions);
+  bl_debug_printf("DND: source_action %d\n", source_actions);
 #endif
 }
 
@@ -1224,7 +1407,7 @@ static void data_offer_action(void *data, struct wl_data_offer *data_offer,
   ui_wlserv_t *wlserv = data;
 
 #ifdef __DEBUG
-  bl_debug_printf("DND ACTION %d\n", dnd_action);
+  bl_debug_printf("DND: action %d\n", dnd_action);
 #endif
 
   wlserv->dnd_action = dnd_action;
@@ -1241,11 +1424,12 @@ static void data_device_data_offer(void *data, struct wl_data_device *data_devic
   ui_wlserv_t *wlserv = data;
 
 #ifdef __DEBUG
-  bl_debug_printf("DND OFFER %p\n", offer);
+  bl_debug_printf("data_device_data_offer %p\n", offer);
 #endif
 
   wl_data_offer_add_listener(offer, &data_offer_listener, wlserv);
   wlserv->dnd_offer = offer;
+  wlserv->sel_offer_mime = NULL;
 }
 
 static void data_device_enter(void *data, struct wl_data_device *data_device,
@@ -1255,7 +1439,7 @@ static void data_device_enter(void *data, struct wl_data_device *data_device,
   ui_wlserv_t *wlserv = data;
 
 #ifdef __DEBUG
-  bl_debug_printf("DATA DEVICE ENTER %p\n", offer);
+  bl_debug_printf("DND: enter %p\n", offer);
 #endif
 
   wl_data_offer_accept(offer, serial, "text/uri-list");
@@ -1273,7 +1457,7 @@ static void data_device_leave(void *data, struct wl_data_device *data_device) {
   ui_wlserv_t *wlserv = data;
 
 #ifdef __DEBUG
-  bl_debug_printf("DATA DEVICE LEAVE\n");
+  bl_debug_printf("DND: leave\n");
 #endif
 
   if (wlserv->dnd_offer) {
@@ -1291,8 +1475,8 @@ static void data_device_motion(void *data, struct wl_data_device *data_device,
                                uint32_t time, wl_fixed_t x_w, wl_fixed_t y_w) {
 }
 
-static int unescape(u_char *src) {
-  u_char *dst;
+static int unescape(char *src) {
+  char *dst;
   int c;
 
   dst = src;
@@ -1310,49 +1494,62 @@ static int unescape(u_char *src) {
   return (src != dst);
 }
 
-static void receive_data(ui_wlserv_t *wlserv, struct wl_data_offer *offer, const char *mime) {
-  ui_display_t *disp;
+static void receive_data(ui_display_t *disp, void *offer, const char *mime, int clipboard) {
   int fds[2];
 
-  if ((disp = surface_to_display(wlserv->data_surface)) == NULL) {
-    return;
-  }
-
   if (pipe(fds) == 0) {
-    u_char buf[512];
+    char buf[512];
     ssize_t len;
 
-    wl_data_offer_receive(offer, mime, fds[1]);
-    wl_display_flush(wlserv->display);
+    if (clipboard) {
+      wl_data_offer_receive(offer, mime, fds[1]);
+    } else {
+      gtk_primary_selection_offer_receive(offer, mime, fds[1]);
+    }
+
+    wl_display_flush(disp->display->wlserv->display);
     close(fds[1]);
 
     while ((len = read(fds[0], buf, sizeof(buf) - 1)) > 0) {
-      char *str;
+      if (strcmp(mime, "text/uri-list") == 0) {
+        char *p;
 
-      while (buf[len - 1] == '\n' || buf[len - 1] == '\r') {
-        len--;
+        while (buf[len - 1] == '\n' || buf[len - 1] == '\r') {
+          len--;
+        }
+        buf[len] = '\0';
+
+        for (p = buf; *p; p++) {
+          if (*p == 'f') {
+            if (strncmp(p + 1, "ile://", 6) == 0) {
+              memmove(p, p + 7, len - 7 + 1);
+              p += 6;
+              len -= 7;
+            }
+          } else if (*p == '\r' || *p == '\n') {
+            *p = ' ';
+          }
+        }
+
+        if (unescape(buf)) {
+          len = strlen(buf);
+        }
       }
-      buf[len] = '\0';
 #ifdef __DEBUG
-      bl_debug_printf("RECEIVE %d %s\n", len, buf);
+      else {
+        buf[len] = '\0';
+      }
+
+      bl_debug_printf("Receive data (len %d, mime %s): %s\n", len, mime, buf);
 #endif
 
-      if (strncmp(buf, "file://", 7) == 0) {
-        str = buf + 7;
-        len -= 7;
-      } else {
-        str = buf;
-      }
-      if (unescape(str)) {
-        len = strlen(str);
-      }
-
-      if (wlserv->dnd_action == WL_DATA_DEVICE_MANAGER_DND_ACTION_MOVE /* Shift+Drop */ &&
+      if (disp->display->wlserv->dnd_action ==
+          WL_DATA_DEVICE_MANAGER_DND_ACTION_MOVE /* Shift+Drop */ &&
           disp->roots[0]->set_xdnd_config) {
-        (*disp->roots[0]->set_xdnd_config)(disp->roots[0], NULL, "scp", str);
+        (*disp->roots[0]->set_xdnd_config)(disp->roots[0], NULL, "scp", buf);
         break;
       } else if (disp->roots[0]->utf_selection_notified) {
-        (*disp->roots[0]->utf_selection_notified)(disp->roots[0], str, len);
+        (*disp->roots[0]->utf_selection_notified)(disp->roots[0], buf, len);
       }
     }
     close(fds[0]);
@@ -1361,25 +1558,31 @@ static void receive_data(ui_wlserv_t *wlserv, struct wl_data_offer *offer, const
 
 static void data_device_drop(void *data, struct wl_data_device *data_device) {
   ui_wlserv_t *wlserv = data;
+  ui_display_t *disp;
 
 #ifdef  __DEBUG
-  bl_debug_printf("DROPPED\n");
+  bl_debug_printf("DND: drop\n");
 #endif
 
-  receive_data(data, wlserv->dnd_offer, "text/uri-list");
+  if ((disp = surface_to_display(wlserv->data_surface))) {
+    receive_data(disp, wlserv->dnd_offer, "text/uri-list", 1);
+  }
 }
 
-static void data_device_selection(void *data, struct wl_data_device *wl_data_device,
+static void data_device_selection(void *data, struct wl_data_device *data_device,
                                   struct wl_data_offer *offer) {
   ui_wlserv_t *wlserv = data;
 
 #ifdef __DEBUG
-  bl_debug_printf("SELECTION OFFER %p\n", offer);
+  bl_debug_printf("data_device_selection %p -> %p\n", wlserv->sel_offer, offer);
 #endif
 
+  if (wlserv->sel_source) {
+    return;
+  }
+
   if (wlserv->sel_offer) {
     wl_data_offer_destroy(wlserv->sel_offer);
-    wlserv->sel_offer = NULL;
   }
   wlserv->sel_offer = offer;
 }
@@ -1402,7 +1605,7 @@ static void data_source_send(void *data, struct wl_data_source *source, const ch
   disp->display->wlserv->sel_fd = fd;
 
 #ifdef __DEBUG
-  bl_debug_printf("SOURCE SEND %s\n", mime);
+  bl_debug_printf("data_source_send %s\n", mime);
 #endif
 
   if (disp->selection_owner->utf_selection_requested) {
@@ -1421,7 +1624,7 @@ static void data_source_cancelled(void *data, struct wl_data_source *source) {
   ui_wlserv_t *wlserv = disp->display->wlserv;
 
 #ifdef __DEBUG
-  bl_debug_printf("SOURCE CANCEL %p %p\n", source, wlserv->sel_source);
+  bl_debug_printf("data_source_cancelled %p %p\n", source, wlserv->sel_source);
 #endif
 
 #ifdef COMPAT_LIBVTE
@@ -1456,6 +1659,100 @@ static const struct wl_data_source_listener data_source_listener = {
   data_source_action,
 };
 
+static void xsel_offer_offer(void *data, struct gtk_primary_selection_offer *offer,
+                                  const char *type) {
+  ui_wlserv_t *wlserv = data;
+
+#ifdef __DEBUG
+  bl_debug_printf("xsel_offer_offer %s\n", type);
+#endif
+
+  update_mime(&wlserv->xsel_offer_mime, type);
+}
+
+static const struct gtk_primary_selection_offer_listener xsel_offer_listener = {
+  xsel_offer_offer,
+};
+
+static void xsel_device_data_offer(void *data,
+                                  struct gtk_primary_selection_device *device,
+                                  struct gtk_primary_selection_offer *offer) {
+  ui_wlserv_t *wlserv = data;
+
+#ifdef __DEBUG
+  bl_debug_printf("gtk_device_data_offer %p\n", offer);
+#endif
+
+  gtk_primary_selection_offer_add_listener(offer, &xsel_offer_listener, wlserv);
+  wlserv->xsel_offer_mime = NULL;
+}
+
+static void xsel_device_selection(void *data,
+                                 struct gtk_primary_selection_device *device,
+                                 struct gtk_primary_selection_offer *offer) {
+  ui_wlserv_t *wlserv = data;
+
+#ifdef __DEBUG
+  bl_debug_printf("gtk_device_selection %p -> %p\n", wlserv->xsel_offer, offer);
+#endif
+
+  if (wlserv->xsel_source) {
+    return;
+  }
+
+  if (wlserv->xsel_offer) {
+    gtk_primary_selection_offer_destroy(wlserv->xsel_offer);
+  }
+  wlserv->xsel_offer = offer;
+}
+
+static const struct gtk_primary_selection_device_listener xsel_device_listener = {
+  xsel_device_data_offer,
+  xsel_device_selection
+};
+
+static void xsel_source_send(void *data, struct gtk_primary_selection_source *source,
+                             const char *mime, int32_t fd) {
+  ui_display_t *disp = data;
+  disp->display->wlserv->sel_fd = fd;
+
+#ifdef __DEBUG
+  bl_debug_printf("xsel_source_send %s\n", mime);
+#endif
+
+  if (disp->selection_owner->utf_selection_requested) {
+    /* utf_selection_requested() calls ui_display_send_text_selection() */
+    (*disp->selection_owner->utf_selection_requested)(disp->selection_owner, NULL, 0);
+  }
+}
+
+static void xsel_source_cancelled(void *data, struct gtk_primary_selection_source *source) {
+  ui_display_t *disp = data;
+  ui_wlserv_t *wlserv = disp->display->wlserv;
+
+#ifdef __DEBUG
+  bl_debug_printf("xsel_source_cancelled %p %p\n", source, wlserv->xsel_source);
+#endif
+
+#ifdef COMPAT_LIBVTE
+  if (deadline_ignoring_source_cancelled_event != 0 &&
+      deadline_ignoring_source_cancelled_event > g_get_monotonic_time()) {
+  } else {
+    deadline_ignoring_source_cancelled_event = 0;
+#endif
+    if (source == wlserv->xsel_source) {
+      ui_display_clear_selection(disp, disp->selection_owner);
+    }
+#ifdef COMPAT_LIBVTE
+  }
+#endif
+}
+
+static const struct gtk_primary_selection_source_listener xsel_source_listener = {
+  xsel_source_send,
+  xsel_source_cancelled,
+};
+
 static ui_wlserv_t *open_wl_display(char *name) {
   ui_wlserv_t *wlserv;
 
@@ -1512,9 +1809,19 @@ static ui_wlserv_t *open_wl_display(char *name) {
     }
   }
 
-  wlserv->data_device = wl_data_device_manager_get_data_device(wlserv->data_device_manager,
-                                                               wlserv->seat);
-  wl_data_device_add_listener(wlserv->data_device, &data_device_listener, wlserv);
+  if (wlserv->data_device_manager) {
+    wlserv->data_device = wl_data_device_manager_get_data_device(wlserv->data_device_manager,
+                                                                 wlserv->seat);
+    wl_data_device_add_listener(wlserv->data_device, &data_device_listener, wlserv);
+  }
+
+  if (wlserv->xsel_device_manager) {
+    wlserv->xsel_device =
+      gtk_primary_selection_device_manager_get_device(wlserv->xsel_device_manager,
+                                                      wlserv->seat);
+    gtk_primary_selection_device_add_listener(wlserv->xsel_device,
+                                              &xsel_device_listener, wlserv);
+  }
 
   wlserv->sel_fd = -1;
 
@@ -1540,10 +1847,26 @@ static void close_wl_display(ui_wlserv_t *wlserv) {
     wl_data_source_destroy(wlserv->sel_source);
     wlserv->sel_source = NULL;
   }
+  if (wlserv->xsel_source) {
+    gtk_primary_selection_source_destroy(wlserv->xsel_source);
+    wlserv->xsel_source = NULL;
+  }
 
 #ifdef COMPAT_LIBVTE
   wl_subcompositor_destroy(wlserv->subcompositor);
+#else
+#ifdef XDG_SHELL_V6
+  if (wlserv->xdg_shell) {
+    zxdg_shell_v6_destroy(wlserv->xdg_shell);
+    wlserv->xdg_shell = NULL;
+  } else if (wlserv->shell)
 #endif
+  {
+    wl_shell_destroy(wlserv->shell);
+    wlserv->shell = NULL;
+  }
+#endif
+
   wl_output_destroy(wlserv->output);
   wl_compositor_destroy(wlserv->compositor);
 #ifndef COMPAT_LIBVTE
@@ -1611,6 +1934,15 @@ static void close_display(ui_display_t *disp) {
     wl_surface_destroy(disp->display->surface);
   }
 #else
+#ifdef XDG_SHELL_V6
+  if (disp->display->xdg_surface) {
+    destroy_shm_buffer(disp->display);
+    disp->display->buffer = NULL;
+    zxdg_toplevel_v6_destroy(disp->display->xdg_toplevel);
+    zxdg_surface_v6_destroy(disp->display->xdg_surface);
+    wl_surface_destroy(disp->display->surface);
+  } else
+#endif
   if (disp->display->shell_surface) {
     destroy_shm_buffer(disp->display);
     disp->display->buffer = NULL;
@@ -1643,8 +1975,27 @@ static void close_display(ui_display_t *disp) {
 
 static int flush_damage(Display *display) {
   if (display->damage_height > 0) {
+    /*
+     * xdg_shell seems to require calling wl_surface_attach() every time
+     * differently from wl_shell.
+     */
+#if defined(COMPAT_LIBVTE)
+    if (display->wlserv->is_xdg_shell == 1) {
+      wl_surface_attach(display->surface, display->buffer, 0, 0);
+    } else
+#elif defined(XDG_SHELL_V6)
+    if (display->xdg_surface) {
+      if (display->xdg_surface_configured) {
+        wl_surface_attach(display->surface, display->buffer, 0, 0);
+      } else {
+        return 0;
+      }
+    }
+#endif
+
     wl_surface_damage(display->surface, display->damage_x, display->damage_y,
                       display->damage_width, display->damage_height);
+
     display->damage_x = display->damage_y = display->damage_width = display->damage_height = 0;
 
     return 1;
@@ -1731,16 +2082,28 @@ static void create_surface(ui_display_t *disp, u_int width, u_int height, char *
   display->subsurface = wl_subcompositor_get_subsurface(wlserv->subcompositor, display->surface,
                                                         display->parent_surface);
 #else
-  display->shell_surface = wl_shell_get_shell_surface(wlserv->shell, display->surface);
-  wl_shell_surface_set_class(display->shell_surface, app_name);
+#ifdef XDG_SHELL_V6
+  if (wlserv->xdg_shell) {
+    display->xdg_surface = zxdg_shell_v6_get_xdg_surface(wlserv->xdg_shell, display->surface);
+    zxdg_surface_v6_add_listener(display->xdg_surface, &xdg_surface_listener, disp);
+    display->xdg_toplevel = zxdg_surface_v6_get_toplevel(display->xdg_surface);
+    zxdg_toplevel_v6_set_app_id(display->xdg_toplevel, app_name);
+    zxdg_toplevel_v6_add_listener(display->xdg_toplevel, &xdg_toplevel_listener, disp);
+    wl_surface_commit(display->surface);
+  } else
+#endif
+  {
+    display->shell_surface = wl_shell_get_shell_surface(wlserv->shell, display->surface);
+    wl_shell_surface_set_class(display->shell_surface, app_name);
 
-  if (!display->parent) {
-    /* Not input method */
+    if (!display->parent) {
+      /* Not input method */
 
-    wl_surface_add_listener(display->surface, &surface_listener, wlserv);
-    wl_shell_surface_add_listener(display->shell_surface, &shell_surface_listener, disp);
-    wl_shell_surface_set_toplevel(display->shell_surface);
-    wl_shell_surface_set_title(display->shell_surface, app_name);
+      wl_surface_add_listener(display->surface, &surface_listener, wlserv);
+      wl_shell_surface_add_listener(display->shell_surface, &shell_surface_listener, disp);
+      wl_shell_surface_set_toplevel(display->shell_surface);
+      wl_shell_surface_set_title(display->shell_surface, app_name);
+    }
   }
 #endif
 
@@ -1838,7 +2201,7 @@ void ui_display_close(ui_display_t *disp) {
         displays[count] = displays[num_displays];
       }
 
-#ifdef DEBUG
+#ifdef __DEBUG
       bl_debug_printf("wayland display connection closed.\n");
 #endif
 
@@ -1918,6 +2281,14 @@ int ui_display_show_root(ui_display_t *disp, ui_window_t *root, int x, int y, in
   } else
 #endif
   {
+#if 0 /* def XDG_SHELL_V6 */
+    if (disp->display->xdg_surface && disp->display->parent == NULL &&
+        (hint & (XValue | YValue))) {
+      zxdg_surface_v6_set_window_geometry(disp->display->xdg_surface,
+                                          x, y, ACTUAL_WIDTH(root), ACTUAL_HEIGHT(root));
+    }
+#endif
+
     create_shm_buffer(disp->display);
 
 #ifndef COMPAT_LIBVTE
@@ -1942,6 +2313,9 @@ int ui_display_show_root(ui_display_t *disp, ui_window_t *root, int x, int y, in
     ui_window_show(root, hint);
   }
 
+  /* XXX mlterm -sbmod=none on sway 1.0 stops without calling wl_display_dispatch() here. */
+  wl_display_dispatch(disp->display->wlserv->display);
+
   return 1;
 }
 
@@ -2091,11 +2465,37 @@ int ui_display_own_selection(ui_display_t *disp, ui_window_t *win) {
     }
   }
 
-  wlserv->sel_source = wl_data_device_manager_create_data_source(wlserv->data_device_manager);
-  wl_data_source_add_listener(wlserv->sel_source, &data_source_listener, disp);
-  wl_data_source_offer(wlserv->sel_source, "UTF8_STRING");
-  wl_data_device_set_selection(wlserv->data_device,
-                               wlserv->sel_source, wlserv->serial);
+  if (wlserv->xsel_device_manager) {
+    wlserv->xsel_source =
+      gtk_primary_selection_device_manager_create_source(wlserv->xsel_device_manager);
+    gtk_primary_selection_source_offer(wlserv->xsel_source, "UTF8_STRING");
+    /* gtk_primary_selection_source_offer(wlserv->xsel_source, "COMPOUND_TEXT"); */
+    gtk_primary_selection_source_offer(wlserv->xsel_source, "TEXT");
+    gtk_primary_selection_source_offer(wlserv->xsel_source, "STRING");
+    gtk_primary_selection_source_offer(wlserv->xsel_source, "text/plain;charset=utf-8");
+    gtk_primary_selection_source_offer(wlserv->xsel_source, "text/plain");
+    gtk_primary_selection_source_add_listener(wlserv->xsel_source,
+                                              &xsel_source_listener, disp);
+    gtk_primary_selection_device_set_selection(wlserv->xsel_device,
+                                               wlserv->xsel_source, wlserv->serial);
+  }
+
+  if (wlserv->data_device_manager) {
+    wlserv->sel_source = wl_data_device_manager_create_data_source(wlserv->data_device_manager);
+    wl_data_source_offer(wlserv->sel_source, "UTF8_STRING");
+    /* wl_data_source_offer(wlserv->sel_source, "COMPOUND_TEXT"); */
+    wl_data_source_offer(wlserv->sel_source, "TEXT");
+    wl_data_source_offer(wlserv->sel_source, "STRING");
+    wl_data_source_offer(wlserv->sel_source, "text/plain;charset=utf-8");
+    wl_data_source_offer(wlserv->sel_source, "text/plain");
+    wl_data_source_add_listener(wlserv->sel_source, &data_source_listener, disp);
+    wl_data_device_set_selection(wlserv->data_device,
+                                 wlserv->sel_source, wlserv->serial);
+  }
+
+#ifdef __DEBUG
+  bl_debug_printf("set_selection\n");
+#endif
 
   return 1;
 }
@@ -2112,6 +2512,10 @@ int ui_display_clear_selection(ui_display_t *disp, /* NULL means all selection o
         wl_data_source_destroy(displays[count]->display->wlserv->sel_source);
         displays[count]->display->wlserv->sel_source = NULL;
       }
+      if (displays[count]->display->wlserv->xsel_source) {
+        gtk_primary_selection_source_destroy(displays[count]->display->wlserv->xsel_source);
+        displays[count]->display->wlserv->xsel_source = NULL;
+      }
     }
   }
 
@@ -2175,6 +2579,12 @@ int ui_display_move(ui_display_t *disp, int x, int y) {
     }
   }
 #else
+#ifdef XDG_SHELL_V6
+  if (display->xdg_surface) {
+    /* XXX */
+    bl_warn_printf("It is impossible to move windows on xdg_shell for now.\n");
+  } else
+#endif
   if (display->parent) {
     /* input method window */
 #ifdef __DEBUG
@@ -2350,14 +2760,21 @@ void ui_display_copy_lines(ui_display_t *disp, int src_x, int src_y, int dst_x,
 void ui_display_request_text_selection(ui_display_t *disp) {
   if (disp->selection_owner) {
     XSelectionRequestEvent ev;
+
     ev.type = 0;
     ev.target = disp->roots[0];
     if (disp->selection_owner->utf_selection_requested) {
       /* utf_selection_requested() calls ui_display_send_text_selection() */
       (*disp->selection_owner->utf_selection_requested)(disp->selection_owner, &ev, 0);
     }
-  } else if (disp->display->wlserv->sel_offer) {
-    receive_data(disp->display->wlserv, disp->display->wlserv->sel_offer, "UTF8_STRING");
+  } else {
+    ui_wlserv_t *wlserv = disp->display->wlserv;
+
+    if (wlserv->xsel_offer && wlserv->xsel_offer_mime) {
+      receive_data(disp, wlserv->xsel_offer, wlserv->xsel_offer_mime, 0);
+    } else if (wlserv->sel_offer && wlserv->sel_offer_mime) {
+      receive_data(disp, wlserv->sel_offer, wlserv->sel_offer_mime, 1);
+    }
   }
 }
 
@@ -2372,15 +2789,9 @@ void ui_display_send_text_selection(ui_display_t *disp, XSelectionRequestEvent *
   }
 }
 
-u_char ui_display_get_char(KeySym ksym) {
-  char buf[10];
-  int len;
-
-  if ((len = xkb_keysym_to_utf8(ksym, buf, sizeof(buf))) > 0) {
-    return buf[0];
-  } else {
-    return 0;
-  }
+size_t ui_display_get_utf8(char *utf8 /* 7 bytes (UTF_MAX_SIZE + 1) */, KeySym ksym) {
+  /* xkb_keysym_to_utf8() appends '\0' to utf8 bytes. */
+  return xkb_keysym_to_utf8(ksym, utf8, 7);
 }
 
 void ui_display_logical_to_physical_coordinates(ui_display_t *disp, int *x, int *y) {
@@ -2404,14 +2815,28 @@ void ui_display_logical_to_physical_coordinates(ui_display_t *disp, int *x, int
 void ui_display_set_maximized(ui_display_t *disp, int flag) {
 #ifndef COMPAT_LIBVTE
   if (flag) {
-    wl_shell_surface_set_maximized(disp->display->shell_surface, disp->display->wlserv->output);
+#ifdef XDG_SHELL_V6
+    if (disp->display->xdg_toplevel) {
+      zxdg_toplevel_v6_set_maximized(disp->display->xdg_toplevel);
+    } else
+#endif
+    {
+      wl_shell_surface_set_maximized(disp->display->shell_surface, disp->display->wlserv->output);
+    }
   }
 #endif
 }
 
 void ui_display_set_title(ui_display_t *disp, const u_char *name) {
 #ifndef COMPAT_LIBVTE
-  wl_shell_surface_set_title(disp->display->shell_surface, name);
+#ifdef XDG_SHELL_V6
+  if (disp->display->xdg_toplevel) {
+    zxdg_toplevel_v6_set_title(disp->display->xdg_toplevel, name);
+  } else
+#endif
+  {
+    wl_shell_surface_set_title(disp->display->shell_surface, name);
+  }
 #endif
 }
 
@@ -2435,9 +2860,19 @@ void ui_display_init_wlserv(ui_wlserv_t *wlserv) {
   wl_pointer_add_listener(wlserv->pointer, &pointer_listener, wlserv);
 #endif
 
-  wlserv->data_device = wl_data_device_manager_get_data_device(wlserv->data_device_manager,
-                                                               wlserv->seat);
-  wl_data_device_add_listener(wlserv->data_device, &data_device_listener, wlserv);
+  if (wlserv->data_device_manager) {
+    wlserv->data_device = wl_data_device_manager_get_data_device(wlserv->data_device_manager,
+                                                                 wlserv->seat);
+    wl_data_device_add_listener(wlserv->data_device, &data_device_listener, wlserv);
+  }
+
+  if (wlserv->xsel_device_manager) {
+    wlserv->xsel_device =
+      gtk_primary_selection_device_manager_get_device(wlserv->xsel_device_manager,
+                                                      wlserv->seat);
+    gtk_primary_selection_device_add_listener(wlserv->xsel_device,
+                                              &xsel_device_listener, wlserv);
+  }
 
   if ((wlserv->cursor_theme = wl_cursor_theme_load(NULL, 32, wlserv->shm))) {
     wlserv->cursor[0] = wl_cursor_theme_get_cursor(wlserv->cursor_theme, "xterm");
