@@ -1,5 +1,7 @@
 /* -*- c-basic-offset:2; tab-width:2; indent-tabs-mode:nil -*- */
 
+#define _GNU_SOURCE /* strcasestr */
+
 #include "ui_font.h"
 
 #include <stdio.h>
@@ -13,7 +15,6 @@
 #include <pobl/bl_mem.h>
 #include <pobl/bl_str.h> /* strdup */
 #include <pobl/bl_debug.h>
-#include <pobl/bl_str.h>    /* strdup */
 #include <pobl/bl_path.h>   /* bl_basename */
 #include <pobl/bl_conf_io.h>/* bl_get_user_rc_path */
 #include <pobl/bl_util.h>   /* TOINT32 */
@@ -25,6 +26,7 @@
 #ifdef USE_WIN32API
 #define mmap(a, b, c, d, e, f) (NULL)
 #define munmap(a, b) (0)
+#define strcasestr(a, b) strstr(a, b)
 #else
 #include <sys/mman.h> /* mmap */
 #endif
@@ -749,11 +751,12 @@ face_found:
   if (force_height) {
     xfont->height = force_height;
   } else {
-    xfont->height = (face->max_advance_height * face->size->metrics.y_ppem +
+    /* XXX face->max_advance_height might be unexpectedly big. (e.g. NotoSansCJKjp-Reguar.otf) */
+    xfont->height = (face->height * face->size->metrics.y_ppem +
                      face->units_per_EM - 1) / face->units_per_EM;
 #ifdef __DEBUG
-    bl_debug_printf("maxh %d ppem %d units %d => h %d\n",
-                    face->max_advance_height, face->size->metrics.y_ppem,
+    bl_debug_printf("height %d (max height %d) ppem %d units %d => h %d\n",
+                    face->height, face->max_advance_height, face->size->metrics.y_ppem,
                     face->units_per_EM, xfont->height);
 #endif
   }
@@ -765,7 +768,7 @@ face_found:
     xfont->width_full = (face->max_advance_width * face->size->metrics.x_ppem +
                          face->units_per_EM - 1) / face->units_per_EM;
 #ifdef __DEBUG
-    bl_debug_printf("maxw %d ppem %d units %d => w %d\n",
+    bl_debug_printf("maxw %d ppem %d units %d => xfont->width_full %d\n",
                     face->max_advance_width, face->size->metrics.x_ppem,
                     face->units_per_EM, xfont->width_full);
 #endif
@@ -782,9 +785,28 @@ face_found:
     if (load_glyph(face, format, get_glyph_index(face, 'j'), is_aa)) {
       int descent = face->glyph->bitmap.rows - face->glyph->bitmap_top;
       if (descent > xfont->height - xfont->ascent) {
+#ifdef __DEBUG
+        bl_debug_printf("Modify xfont->height to %d\n", xfont->ascent + descent);
+#endif
         xfont->height = xfont->ascent + descent;
       }
     }
+
+    if (load_glyph(face, format, get_glyph_index(face, 'W'), is_aa)) {
+      u_int w;
+      if (is_aa) {
+        w = face->glyph->bitmap.width / 3;
+      } else {
+        w = face->glyph->bitmap.width;
+      }
+
+      if (xfont->width_full < w) {
+#ifdef __DEBUG
+        bl_debug_printf("Modify xfont->width_full to %d\n", w);
+#endif
+        xfont->width_full = w;
+      }
+    }
   }
 
   if (is_aa) {
@@ -1119,11 +1141,30 @@ static u_char *get_ft_bitmap_intern(XFontStruct *xfont, u_int32_t code /* glyph
           dst += (glyph[2] * 3);
         }
       } else {
-        for (y = 0; y < rows; y++) {
-          memcpy(dst + left_pitch, src, pitch);
+        if (face->glyph->bitmap_left < 0 && left_pitch == 0 &&
+            xfont->width * 3 < face->glyph->bitmap.width /* pitch? */) {
+          /* For 'A' and 'W' (bitmap_left can be -1) of Incosnolata font */
+          int left_pitch_minus = face->glyph->bitmap_left * 3;
 
-          src += face->glyph->bitmap.pitch;
-          dst += xfont->glyph_width_bytes;
+          if (pitch + left_pitch_minus < 0) {
+            left_pitch_minus = -pitch;
+          }
+
+          src -= left_pitch_minus;
+          for (y = 0; y < rows - 1; y++) {
+            memcpy(dst, src, pitch);
+
+            src += face->glyph->bitmap.pitch;
+            dst += xfont->glyph_width_bytes;
+          }
+          memcpy(dst, src, pitch + left_pitch_minus);
+        } else {
+          for (y = 0; y < rows; y++) {
+            memcpy(dst + left_pitch, src, pitch);
+
+            src += face->glyph->bitmap.pitch;
+            dst += xfont->glyph_width_bytes;
+          }
         }
       }
     } else {
@@ -1140,6 +1181,7 @@ static u_char *get_ft_bitmap_intern(XFontStruct *xfont, u_int32_t code /* glyph
           dst += xfont->glyph_width_bytes;
         }
       } else {
+        /* XXX TODO: Support face->glyph->bitmap_left < 0 */
         int count;
         for (y = 0; y < rows; y++) {
           dst[0] = (src[0] >> left_pitch);
@@ -1210,7 +1252,7 @@ static void compl_final(void) {
   }
 }
 
-static void compl_xfonts_delete(XFontStruct **xfonts) {
+static void compl_xfonts_destroy(XFontStruct **xfonts) {
   if (xfonts) {
     u_int count;
 
@@ -1241,6 +1283,18 @@ static int parse_fc_font_name(char **font_family,
 
   *font_family = font_name;
 
+  p = font_name + strlen(font_name) - 1;
+  if ('0' <= *p && *p <= '9') {
+    do {
+      p--;
+    } while ('0' <= *p && *p <= '9');
+
+    if (*p == ':' && bl_str_to_uint(percent, p + 1)) {
+      /* Parsing ":[Percentage]" */
+      *p = '\0';
+    }
+  }
+
   p = font_name;
   while (1) {
     if (*p == '\\' && *(p + 1)) {
@@ -1253,18 +1307,6 @@ static int parse_fc_font_name(char **font_family,
 
       *font_name = '\0';
 
-      break;
-    } else if (*p == ':') {
-      /* Parsing ":[Percentage]" */
-
-      *font_name = '\0';
-
-      if (!bl_str_to_uint(percent, p + 1)) {
-#ifdef DEBUG
-        bl_warn_printf(BL_DEBUG_TAG " Percentage(%s) is illegal.\n", p + 1);
-#endif
-      }
-
       break;
     }
 
@@ -1396,14 +1438,16 @@ static u_int strip_pattern(FcPattern *pattern, FcPattern *remove) {
 static FcPattern *fc_pattern_create(const FcChar8* family) {
   FcPattern *pattern;
 
-  if (!(pattern = FcPatternCreate())) {
+  if (family && (pattern = FcNameParse(family))) {
+    /* do nothing */
+  } else if ((pattern = FcPatternCreate())) {
+    if (family) {
+      FcPatternAddString(pattern, FC_FAMILY, family);
+    }
+  } else {
     return NULL;
   }
 
-  if (family) {
-    FcPatternAddString(pattern, FC_FAMILY, family);
-  }
-
   FcConfigSubstitute(NULL, pattern, FcMatchPattern);
 
   FcPatternRemove(pattern, FC_FAMILYLANG, 0);
@@ -1444,7 +1488,11 @@ static FcPattern *parse_font_name(const char *fontname, int *is_bold, int *is_it
   if (!fontname) {
     family = NULL;
   } else {
-    parse_fc_font_name(&family, is_bold, is_italic, percent, bl_str_alloca_dup(fontname));
+    char *p;
+
+    if ((p = alloca(strlen(fontname) + 1))) {
+      parse_fc_font_name(&family, is_bold, is_italic, percent, strcpy(p, fontname));
+    }
   }
 
   if ((pattern = fc_pattern_create(family))) {
@@ -1639,7 +1687,7 @@ static u_char *get_ft_bitmap(XFontStruct *xfont, u_int32_t ch, int use_ot_layout
             *compl_xfont = compl;
           }
 #ifdef __DEBUG
-          bl_debug_printf("Use %s font to show U+%x\n", (*compl_xfont)->file, ch);
+          bl_debug_printf("Use %s font to show U+%x\n", compl->file, ch);
 #endif
 
           return bitmap;
@@ -1704,7 +1752,7 @@ static void xfont_unref(XFontStruct *xfont) {
     u_int count;
 
 #if defined(USE_FREETYPE) && defined(USE_FONTCONFIG)
-    compl_xfonts_delete(xfont->compl_xfonts);
+    compl_xfonts_destroy(xfont->compl_xfonts);
 #endif
 
     for (count = 0; count < num_xfonts; count++) {
@@ -1772,6 +1820,7 @@ ui_font_t *ui_font_new(Display *display, vt_font_t id, int size_attr, ui_type_en
     FcValue val;
     int is_bold;
     int is_italic;
+    char *p;
 
 #ifdef USE_WIN32API
     init_fontconfig();
@@ -1781,20 +1830,30 @@ ui_font_t *ui_font_new(Display *display, vt_font_t id, int size_attr, ui_type_en
       return NULL;
     }
 
-    if (is_bold) {
-      id |= FONT_BOLD;
-    }
-    if (is_italic) {
-      id |= FONT_ITALIC;
-    }
-
     if (FcPatternGet(pattern, FC_FILE, 0, &val) != FcResultMatch) {
       FcPatternDestroy(pattern);
 
       return NULL;
     }
 
-    font_file = bl_str_alloca_dup(val.u.s);
+    if ((font_file = alloca(strlen(val.u.s) + 1))) {
+      strcpy(font_file, val.u.s);
+    }
+
+    /* For 'format = fontsize | (id & (FONT_BOLD | FONT_ITALIC))' below */
+    if ((p = strcasestr(font_file, "bold")) &&
+        strchr(p, '/') == NULL && strchr(p, '\\') == NULL) {
+      id &= ~FONT_BOLD; /* XXX e.g. Inconsolata-Bold.ttf => Don't call FT_Outline_Embolden(). */
+    } else if (is_bold) {
+      id |= FONT_BOLD;
+    }
+    if ((p = strcasestr(font_file, "italic")) &&
+        strchr(p, '/') == NULL && strchr(p, '\\') == NULL) {
+      id &= ~FONT_ITALIC; /* XXX */
+    } else if (is_italic) {
+      id |= FONT_ITALIC;
+    }
+
     FcPatternDestroy(pattern);
   } else
 #endif
@@ -1860,9 +1919,13 @@ ui_font_t *ui_font_new(Display *display, vt_font_t id, int size_attr, ui_type_en
         while ((entry = readdir(dir))) {
           if (strcasestr(entry->d_name, ".tt")) {
             if (cand == NULL) {
-              cand = bl_str_alloca_dup(entry->d_name);
+              if ((cand = alloca(strlen(entry->d_name) + 1))) {
+                strcpy(cand, entry->d_name);
+              }
             } else if (strcasestr(entry->d_name, "Mono")) {
-              cand = bl_str_alloca_dup(entry->d_name);
+              if ((cand = alloca(strlen(entry->d_name) + 1))) {
+                strcpy(cand, entry->d_name);
+              }
               break;
             }
           }
@@ -1899,9 +1962,10 @@ ui_font_t *ui_font_new(Display *display, vt_font_t id, int size_attr, ui_type_en
   } else {
     char *percent_str;
 
-    if (!(percent_str = bl_str_alloca_dup(fontname))) {
+    if (!(percent_str = alloca(strlen(fontname) + 1))) {
       return NULL;
     }
+    strcpy(percent_str, fontname);
 
 #ifdef USE_WIN32API
     if (percent_str[0] != '\0' && percent_str[1] == ':') {
@@ -2218,9 +2282,9 @@ xfont_loaded:
     font->ascent = fontsize;
   }
 
+  bl_msg_printf("Load %s (id %x)\n", font->xfont->file, font->id);
 #ifdef DEBUG
-  bl_debug_printf(BL_DEBUG_TAG " %s font is loaded. => CURRENT NUM OF XFONTS %d\n",
-                  font->xfont->file, num_xfonts);
+  bl_debug_printf(" => CURRENT NUM OF XFONTS %d\n", num_xfonts);
 #endif
 
 #ifdef DEBUG
@@ -2230,7 +2294,7 @@ xfont_loaded:
   return font;
 }
 
-void ui_font_delete(ui_font_t *font) {
+void ui_font_destroy(ui_font_t *font) {
   xfont_unref(font->xfont);
 
 #ifdef USE_OT_LAYOUT
