@@ -74,6 +74,7 @@ static ef_parser_t *m_cp_parser;
 static LONG decorate_width;
 static LONG decorate_height; /* Height of Title bar etc. */
 static int use_urgent_bell;
+static int pointer_hidden;
 
 /* --- static functions --- */
 
@@ -388,7 +389,7 @@ static u_int total_min_height(ui_window_t *win) {
   return min_height;
 }
 
-static u_int total_width_inc(ui_window_t *win) {
+static u_int max_width_inc(ui_window_t *win) {
   u_int count;
   u_int width_inc;
 
@@ -403,7 +404,7 @@ static u_int total_width_inc(ui_window_t *win) {
        * XXX
        * we should calculate least common multiple of width_inc and sub_inc.
        */
-      if ((sub_inc = total_width_inc(win->children[count])) > width_inc) {
+      if ((sub_inc = max_width_inc(win->children[count])) > width_inc) {
         width_inc = sub_inc;
       }
     }
@@ -412,7 +413,7 @@ static u_int total_width_inc(ui_window_t *win) {
   return width_inc;
 }
 
-static u_int total_height_inc(ui_window_t *win) {
+static u_int max_height_inc(ui_window_t *win) {
   u_int count;
   u_int height_inc;
 
@@ -427,7 +428,7 @@ static u_int total_height_inc(ui_window_t *win) {
        * XXX
        * we should calculate least common multiple of width_inc and sub_inc.
        */
-      if ((sub_inc = total_height_inc(win->children[count])) > height_inc) {
+      if ((sub_inc = max_height_inc(win->children[count])) > height_inc) {
         height_inc = sub_inc;
       }
     }
@@ -904,6 +905,20 @@ int ui_window_set_transparent(ui_window_t *win, ui_picture_modifier_t *pic_mod)
 int ui_window_unset_transparent(ui_window_t *win) { return 0; }
 
 void ui_window_set_cursor(ui_window_t *win, u_int cursor_shape) {
+  if (win->my_window != None) {
+    if (cursor_shape == XC_nil) {
+      if (!pointer_hidden) {
+        ShowCursor(FALSE);
+        pointer_hidden = 1;
+      }
+    } else if (win->cursor_shape == XC_nil) {
+      if (pointer_hidden) {
+        ShowCursor(TRUE);
+        pointer_hidden = 0;
+      }
+    }
+  }
+
   win->cursor_shape = cursor_shape;
 }
 
@@ -1958,6 +1973,11 @@ int ui_window_receive_event(ui_window_t *win, XEvent *event) {
       return 1;
 
     case WM_MOUSEMOVE:
+      if (pointer_hidden) {
+        ShowCursor(TRUE);
+        pointer_hidden = 0;
+      }
+
       if (win->button_is_pressing ||
           ((win->event_mask & PointerMotionMask) && win->pointer_motion)) {
         XMotionEvent mev;
@@ -2119,8 +2139,8 @@ int ui_window_receive_event(ui_window_t *win, XEvent *event) {
           if (IsZoomed(ui_get_root_window(win)->my_window)) {
             width_surplus = height_surplus = 0;
           } else {
-            width_surplus = (width - min_width - win->hmargin * 2) % total_width_inc(win);
-            height_surplus = (height - min_height - win->vmargin * 2) % total_height_inc(win);
+            width_surplus = (width - min_width - win->hmargin * 2) % max_width_inc(win);
+            height_surplus = (height - min_height - win->vmargin * 2) % max_height_inc(win);
           }
 
           win->width = width - win->hmargin * 2;
