@@ -18,10 +18,6 @@
 #include <mef/ef_utf8_conv.h>
 #include <mef/ef_utf16_parser.h>
 
-#if 0
-#define SELF_TEST
-#endif
-
 #define SUCCESS 0
 #define FAILURE -1
 
@@ -431,6 +427,10 @@ static int parse(ui_window_t *win, u_char *src, int len) {
 
   if (!(win->dnd->waiting_atom)) return FAILURE;
 
+  if (win->dnd->action == XInternAtom(win->disp->display, "XdndActionMove", False)) {
+    return parse_text_uri_list(win, src, len);
+  }
+
   for (proc_entry = dnd_parsers; proc_entry->atomname; proc_entry++) {
     if ((win->dnd->waiting_atom) == XInternAtom(win->disp->display, proc_entry->atomname, False))
       break;
@@ -807,8 +807,7 @@ static void TEST_parse_text_uri_list(ui_display_t *disp) {
   int count;
 
   memset(&win, 0, sizeof(win));
-  /* disp is used in l.231: XInternAtom( win->disp->display, "XdndActionMove",
-   * False) */
+  /* disp is used for XInternAtom(win->disp->display, "XdndActionMove", False) */
   win.disp = disp;
   win.utf_selection_notified = TEST_parse_text_uri_list_utf_selection_notified;
   memset(&dnd, 0, sizeof(dnd));
@@ -820,7 +819,11 @@ static void TEST_parse_text_uri_list(ui_display_t *disp) {
   }
 }
 
-static void TEST_dnd(ui_display_t *disp) { TEST_parse_text_uri_list(disp); }
+static void TEST_dnd(ui_display_t *disp) {
+  TEST_parse_text_uri_list(disp);
+
+  bl_msg_printf("PASS X DnD test.\n");
+}
 
 #endif
 
