@@ -11,6 +11,7 @@
 
 #include <stdlib.h>    /* abs */
 #include <string.h>    /* memset/memcpy */
+#include <unistd.h>    /* getpid */
 #include <X11/Xutil.h> /* for XSizeHints */
 #include <X11/Xatom.h>
 #include <pobl/bl_debug.h>
@@ -56,6 +57,7 @@
  * Extended Window Manager Hint support
  */
 #define XA_NET_WM_ICON(display) (XInternAtom(display, "_NET_WM_ICON", False))
+#define XA_NET_WM_PID(display) (XInternAtom(display, "_NET_WM_PID", False))
 
 /*
  * Motif Window Manager Hint (for borderless window)
@@ -497,15 +499,7 @@ static u_int total_width_inc(ui_window_t *win) {
   for (count = 0; count < win->num_children; count++) {
     if (win->children[count]->is_mapped &&
         (win->children[count]->sizehint_flag & SIZEHINT_WIDTH)) {
-      u_int sub_inc;
-
-      /*
-       * XXX
-       * we should calculate least common multiple of width_inc and sub_inc.
-       */
-      if ((sub_inc = total_width_inc(win->children[count])) > width_inc) {
-        width_inc = sub_inc;
-      }
+      width_inc += total_width_inc(win->children[count]);
     }
   }
 
@@ -521,13 +515,53 @@ static u_int total_height_inc(ui_window_t *win) {
   for (count = 0; count < win->num_children; count++) {
     if (win->children[count]->is_mapped &&
         (win->children[count]->sizehint_flag & SIZEHINT_HEIGHT)) {
+      height_inc += total_height_inc(win->children[count]);
+    }
+  }
+
+  return height_inc;
+}
+
+static u_int max_width_inc(ui_window_t *win) {
+  u_int count;
+  u_int width_inc;
+
+  width_inc = win->width_inc;
+
+  for (count = 0; count < win->num_children; count++) {
+    if (win->children[count]->is_mapped &&
+        (win->children[count]->sizehint_flag & SIZEHINT_WIDTH)) {
       u_int sub_inc;
 
       /*
        * XXX
        * we should calculate least common multiple of width_inc and sub_inc.
        */
-      if ((sub_inc = total_height_inc(win->children[count])) > height_inc) {
+      if ((sub_inc = total_width_inc(win->children[count])) > width_inc) {
+        width_inc = sub_inc;
+      }
+    }
+  }
+
+  return width_inc;
+}
+
+static u_int max_height_inc(ui_window_t *win) {
+  u_int count;
+  u_int height_inc;
+
+  height_inc = win->height_inc;
+
+  for (count = 0; count < win->num_children; count++) {
+    if (win->children[count]->is_mapped &&
+        (win->children[count]->sizehint_flag & SIZEHINT_HEIGHT)) {
+      u_int sub_inc;
+
+      /*
+       * XXX
+       * we should calculate least common multiple of width_inc and sub_inc.
+       */
+      if ((sub_inc = max_height_inc(win->children[count])) > height_inc) {
         height_inc = sub_inc;
       }
     }
@@ -1314,6 +1348,7 @@ int ui_window_show(ui_window_t *win, int hint) {
     /* Root window */
 
     XSizeHints size_hints;
+    int total;
     XClassHint class_hint;
     XWMHints wm_hints;
     int argc = 1;
@@ -1321,6 +1356,7 @@ int ui_window_show(ui_window_t *win, int hint) {
         "mlterm", NULL,
     };
     Atom protocols[2];
+    XID pid;
 
     win->event_mask |= StructureNotifyMask;
 
@@ -1333,16 +1369,16 @@ int ui_window_show(ui_window_t *win, int hint) {
     size_hints.width = ACTUAL_WIDTH(win);
     size_hints.height = ACTUAL_HEIGHT(win);
 
-    size_hints.width_inc = total_width_inc(win);
-    size_hints.height_inc = total_height_inc(win);
+    size_hints.width_inc = max_width_inc(win);
+    size_hints.height_inc = max_height_inc(win);
     size_hints.min_width = total_min_width(win);
     size_hints.min_height = total_min_height(win);
-    size_hints.base_width = (size_hints.width_inc > 0 &&
-                             size_hints.min_width > size_hints.width_inc) ?
-                            size_hints.min_width % size_hints.width_inc : 0;
-    size_hints.base_height = (size_hints.height_inc > 0 &&
-                              size_hints.min_height > size_hints.height_inc) ?
-                             size_hints.min_height % size_hints.height_inc : 0;
+    total = total_width_inc(win);
+    size_hints.base_width = (size_hints.min_width > total) ?
+                             size_hints.min_width - total : 0;
+    total = total_height_inc(win);
+    size_hints.base_height = (size_hints.min_height > total) ?
+                              size_hints.min_height - total : 0;
 
 #ifdef DEBUG
     bl_debug_printf(BL_DEBUG_TAG " Size hints => w %d h %d wi %d hi %d mw %d mh %d bw %d bh %d\n",
@@ -1372,7 +1408,12 @@ int ui_window_show(ui_window_t *win, int hint) {
       size_hints.flags |= USPosition;
     }
 
-    class_hint.res_name = win->app_name;
+    /*
+     * XXX
+     * If res_name = "mlterm" and res_class = "mlterm", lxde (0.99.2) doesn't
+     * show title bar for mlterm.
+     */
+    class_hint.res_name = "xterm"; /* same as xterm */
     class_hint.res_class = win->app_name;
 
     wm_hints.initial_state = NormalState; /* or IconicState */
@@ -1383,23 +1424,19 @@ int ui_window_show(ui_window_t *win, int hint) {
     bl_debug_printf(BL_DEBUG_TAG " Group leader -> %x\n", wm_hints.window_group);
 #endif
 
-    /*
-     * Notify to window manager.
-     *
-     * XXX If class_hint is set, lxde (0.99.2) doesn't show title bar for mlterm.
-     */
-#if 0
+    /* Notify to window manager. */
     XmbSetWMProperties(win->disp->display, win->my_window, win->app_name, win->app_name, argv, argc,
                        &size_hints, &wm_hints, &class_hint);
-#else
-    XmbSetWMProperties(win->disp->display, win->my_window, win->app_name, win->app_name, argv, argc,
-                       &size_hints, &wm_hints, NULL);
-#endif
 
     protocols[0] = XA_DELETE_WINDOW(win->disp->display);
     protocols[1] = XA_TAKE_FOCUS(win->disp->display);
 
     XSetWMProtocols(win->disp->display, win->my_window, protocols, 2);
+
+    pid = getpid();
+    XChangeProperty(win->disp->display, win->my_window,
+                    XA_NET_WM_PID(win->disp->display), XA_CARDINAL,
+                    32, PropModeReplace, (unsigned char *)&pid, 1);
   }
 
   if (win->parent && !win->parent->is_transparent && win->parent->wall_picture_is_set) {
@@ -1572,6 +1609,7 @@ void ui_window_set_maximize_flag(ui_window_t *win, ui_maximize_flag_t flag) {
 void ui_window_set_normal_hints(ui_window_t *win, u_int min_width, u_int min_height,
                                 u_int width_inc, u_int height_inc) {
   XSizeHints size_hints;
+  int total;
   ui_window_t *root;
 
   win->min_width = min_width;
@@ -1584,16 +1622,16 @@ void ui_window_set_normal_hints(ui_window_t *win, u_int min_width, u_int min_hei
   /*
    * these hints must be set at the same time !
    */
-  size_hints.width_inc = total_width_inc(root);
-  size_hints.height_inc = total_height_inc(root);
+  size_hints.width_inc = max_width_inc(root);
+  size_hints.height_inc = max_height_inc(root);
   size_hints.min_width = total_min_width(root);
   size_hints.min_height = total_min_height(root);
-  size_hints.base_width = (size_hints.width_inc > 0 &&
-                           size_hints.min_width > size_hints.width_inc) ?
-                          size_hints.min_width % size_hints.width_inc : 0;
-  size_hints.base_height = (size_hints.height_inc > 0 &&
-                            size_hints.min_height > size_hints.height_inc) ?
-                           size_hints.min_height % size_hints.height_inc : 0;
+  total = total_width_inc(win);
+  size_hints.base_width = (size_hints.min_width > total) ?
+                           size_hints.min_width - total : 0;
+  total = total_height_inc(win);
+  size_hints.base_height = (size_hints.min_height > total) ?
+                            size_hints.min_height - total : 0;
   size_hints.flags = PMinSize | PResizeInc | PBaseSize;
 
 #ifdef DEBUG
