@@ -30,7 +30,7 @@ typedef enum vt_ctl_bidi_id {
 typedef enum vt_ctl_iscii_id {
   CTL_ISCII_API_COMPAT_CHECK,
   VT_ISCIIKEY_STATE_NEW,
-  VT_ISCIIKEY_STATE_DELETE,
+  VT_ISCIIKEY_STATE_DESTROY,
   VT_CONVERT_ASCII_TO_ISCII,
   VT_LINE_SET_USE_ISCII,
   VT_LINE_ISCII_CONVERT_LOGICAL_CHAR_INDEX_TO_VISUAL,
