@@ -551,7 +551,7 @@ pid_t vt_term_get_pty_mode(vt_term_t *term) {
 }
 
 size_t vt_term_write(vt_term_t *term, u_char *buf, size_t len) {
-  if (term->pty == NULL) {
+  if (term->pty == NULL || vt_parser_is_transferring_data(term->parser)) {
     return 0;
   }
 
@@ -564,8 +564,10 @@ size_t vt_term_write(vt_term_t *term, u_char *buf, size_t len) {
 
 /* The caller should swap width_pix and height_pix in vertical mode. */
 int vt_term_resize(vt_term_t *term, u_int cols, u_int rows, u_int width_pix, u_int height_pix) {
+  int ret;
+
   vt_screen_logical(term->screen);
-  vt_screen_resize(term->screen, cols, rows);
+  ret = vt_screen_resize(term->screen, cols, rows, 1);
   vt_screen_render(term->screen);
   vt_screen_visual(term->screen);
 
@@ -575,7 +577,7 @@ int vt_term_resize(vt_term_t *term, u_int cols, u_int rows, u_int width_pix, u_i
                        vt_screen_get_logical_rows(term->screen), width_pix, height_pix);
   }
 
-  return 1;
+  return ret;
 }
 
 int vt_term_unhighlight_cursor(vt_term_t *term, int revert_visual) {
@@ -964,6 +966,7 @@ int vt_term_get_config(vt_term_t *term, vt_term_t *output, /* if term == output,
   return 1;
 }
 
+/* Called in visual context */
 int vt_term_set_config(vt_term_t *term, char *key, char *value) {
   if (vt_parser_set_config(term->parser, key, value)) {
     /* do nothing */
