@@ -200,13 +200,13 @@ void vt_term_manager_final(void) {
     /*
      * All windows may be invalid before vt_term_manager_final() is called.
      * Without this vt_term_detach(), if terms[count] is not detached,
-     * pty_listener::pty_closed() which is called in vt_pty_delete() can
+     * pty_listener::pty_closed() which is called in vt_pty_destroy() can
      * operate invalid window.
      */
     vt_term_detach(terms[count]);
 #endif
 
-    vt_term_delete(terms[count]);
+    vt_term_destroy(terms[count]);
   }
 
   free(terms);
@@ -216,7 +216,7 @@ void vt_term_manager_final(void) {
   free(auto_restart_cmd);
 }
 
-void vt_set_auto_restart_cmd(char *cmd) {
+void vt_set_auto_restart_cmd(const char *cmd) {
 #if !defined(USE_WIN32API) && !defined(DEBUG)
   char *env;
 
@@ -272,6 +272,7 @@ vt_term_t *vt_create_term(const char *term_type, u_int cols, u_int rows, u_int t
                           int ignore_broadcasted_chars) {
 #if !defined(USE_WIN32API) && !defined(DEBUG)
   char *list;
+  char *list_tmp;
 #endif
 
   if (num_terms == MAX_TERMS) {
@@ -279,12 +280,13 @@ vt_term_t *vt_create_term(const char *term_type, u_int cols, u_int rows, u_int t
   }
 
 #if !defined(USE_WIN32API) && !defined(DEBUG)
-  if ((list = getenv("INHERIT_PTY_LIST")) && (list = bl_str_alloca_dup(list))) {
+  if ((list_tmp = getenv("INHERIT_PTY_LIST")) && (list = alloca(strlen(list_tmp) + 1))) {
     int master;
     int slave;
     pid_t child_pid;
     char *p;
 
+    strcpy(list, list_tmp);
     while ((p = bl_str_sep(&list, ","))) {
       vt_pty_ptr_t pty;
 
@@ -304,7 +306,7 @@ vt_term_t *vt_create_term(const char *term_type, u_int cols, u_int rows, u_int t
 
             continue;
           } else {
-            vt_pty_delete(pty);
+            vt_pty_destroy(pty);
           }
         }
 
@@ -367,7 +369,7 @@ void vt_destroy_term(vt_term_t *term) {
     }
   }
 
-  vt_term_delete(term);
+  vt_term_destroy(term);
 }
 
 vt_term_t *vt_get_term(const char *dev) {
@@ -484,7 +486,7 @@ void vt_close_dead_terms(void) {
             term = terms[idx * MTU + count];
             /*
              * Update terms and num_terms before
-             * vt_term_delete, which calls
+             * vt_term_destroy, which calls
              * vt_pty_event_listener::pty_close in which
              * vt_term_manager can be used.
              */
@@ -492,7 +494,7 @@ void vt_close_dead_terms(void) {
             if (zombie_pty) {
               vt_term_zombie(term);
             } else {
-              vt_term_delete(term);
+              vt_term_destroy(term);
             }
           }
         }
@@ -533,3 +535,19 @@ char *vt_get_pty_list(void) {
 }
 
 void vt_term_manager_enable_zombie_pty(void) { zombie_pty = 1; }
+
+#if defined(__HAIKU__) || defined(__APPLE__) /* only for cocoatouch */
+int vt_check_sig_child(void) {
+  if (num_terms > 0) {
+    int idx;
+
+    for (idx = (num_terms - 1) / MTU; idx >= 0; idx--) {
+      if (dead_mask[idx]) {
+        return 1;
+      }
+    }
+  }
+
+  return 0;
+}
+#endif
